############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################

# import basic stuff
import logging
import PyQt5
import time
from win32com.client.dynamic import Dispatch
import pythoncom


class Weather(PyQt5.QtCore.QThread):
    logger = logging.getLogger(__name__)                                                                                    # get logger for  problems
    signalWeatherData = PyQt5.QtCore.pyqtSignal([dict], name='weatherData')                                                       # single for data transfer to gui
    signalWeatherConnected = PyQt5.QtCore.pyqtSignal([int], name='weatherConnected')                                              # signal for connection status

    def __init__(self, app):
        super().__init__()
        self.app = app
        self.connected = 2
        self.ascom = None                                                                                                   # placeholder for ascom driver object
        self.chooser = None                                                                                                 # placeholder for ascom chooser object
        self.driverName = ''                                                                                                # driver object name
        self.slewing = False
        self.counter = 0
        self.initConfig()

    def initConfig(self):
        try:
            if 'ASCOMWeatherDriverName' in self.app.config:
                self.driverName = self.app.config['ASCOMWeatherDriverName']
        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass

    def storeConfig(self):
        self.app.config['ASCOMWeatherDriverName'] = self.driverName

    def run(self):                                                                                                          # runnable for doing the work
        pythoncom.CoInitialize()                                                                                            # needed for doing CO objects in threads
        self.connected = 0                                                                                                  # set connection flag for stick itself
        self.counter = 0
        while True:                                                                                                         # main loop for stick thread
            self.signalWeatherConnected.emit(self.connected)                                                                # send status to GUI
            if self.connected == 1:                                                                                         # differentiate between dome connected or not
                if self.counter == 0:                                                                                       # jobs once done at the beginning
                    self.getStatusOnce()                                                                                    # task once
                if self.counter % 2 == 0:                                                                                   # all tasks with 200 ms
                    self.getStatusFast()                                                                                    # polling the mount status Ginfo
                if self.counter % 20 == 0:                                                                                  # all tasks with 3 s
                    self.getStatusMedium()                                                                                  # polling the mount
                if self.counter % 300 == 0:                                                                                 # all task with 1 minute
                    self.getStatusSlow()                                                                                    # slow ones
                self.counter += 1                                                                                           # increasing counter for selection
                time.sleep(.1)
            else:
                try:
                    if self.driverName == '':
                        self.connected = 2
                    else:
                        self.ascom = Dispatch(self.driverName)                                                              # load driver
                        self.ascom.connected = True
                        self.connected = 1                                                                                  # set status to connected
                        self.logger.info('driver chosen:{0}'.format(self.driverName))
                except Exception as e:                                                                                      # if general exception
                    if self.driverName != '':
                        self.logger.error('general exception: {0}'.format(e))                             # write to logger
                    if self.driverName == '':
                        self.connected = 2
                    else:
                        self.connected = 0                                                                                  # run the driver setup dialog
                finally:                                                                                                    # still continua and try it again
                    pass                                                                                                    # needed for continue
                time.sleep(5)                                                                                               # wait for the next cycle
        self.ascom.Quit()
        pythoncom.CoUninitialize()                                                                                          # needed for doing COm objects in threads
        self.terminate()                                                                                                    # closing the thread at the end

    def __del__(self):                                                                                                      # remove thread
        self.wait()

    def getStatusFast(self):
        pass

    def getStatusMedium(self):
        data = dict()
        try:
            data['DewPoint'] = self.ascom.DewPoint
            data['Temperature'] = self.ascom.Temperature
            data['Humidity'] = self.ascom.Humidity
            data['Pressure'] = self.ascom.Pressure
            data['CloudCover'] = self.ascom.CloudCover
            data['RainRate'] = self.ascom.RainRate
            data['WindSpeed'] = self.ascom.WindSpeed
            data['WindDirection'] = self.ascom.WindDirection
            self.signalWeatherData.emit(data)                                                                               # send data
        except Exception as e:
            self.logger.error('error accessing weather ascom data: {}'.format(e))

    def getStatusSlow(self):
        pass

    def getStatusOnce(self):
        pass

    def setupDriver(self):  #
        try:
            if self.ascom:
                self.ascom.connected = False
            self.chooser = Dispatch('ASCOM.Utilities.Chooser')
            self.chooser.DeviceType = 'ObservingConditions'
            self.driverName = self.chooser.Choose(self.driverName)
            self.logger.info('driver chosen:{0}'.format(self.driverName))
            if self.driverName == '':
                self.connected = 2
            else:
                self.connected = 0                                                                                          # run the driver setup dialog
        except Exception as e:                                                                                              # general exception
            self.app.messageQueue.put('Driver Exception in setupWeather')                                                   # write to gui
            self.logger.error('general exception:{0}'.format(e))
            if self.driverName == '':
                self.connected = 2
            else:
                self.connected = 0                                                                                          # run the driver setup dialog
        finally:                                                                                                            # continue to work
            pass                                                                                                            # python necessary



#!/usr/bin/env python
"""
A very basic fits file parser. This only handles fits file
that contain a single (gray-scale) image. It was designed
primarily for the purpose of handling images from an 
INDI server.

Hazen 11/16
"""

import numpy


class SimpleFitsException(Exception):
    pass


def parseValue(string):
    """
    Try to convert a numeric string to an integer or a float.
    """
    try:
        return int(string)
    except ValueError:
        try:
            return float(string)
        except ValueError:
            return string[1:-1]

        
class FitsImage(object):

    def __init__(self, fits_name = None, fits_string = None, verbose = True):
        self.keywords = {}
        self.np_data = None

#        if fits_name is None and fits_string is None:
#            raise SimpleFitsException("Must specify a fits file or a string containing a fits images.")

        if fits_name is not None:
            with open(fits_name, "rb") as fp:
                fits_string = fp.read()

        # Read out keywords:
        nkw = 0
        while True:

            record = fits_string[nkw*80:(nkw+1)*80]
            nkw += 1
            if verbose:
                print(record)

            if record.startswith(b'END'):
                break

            if record.startswith(b'COMMENT'):
                continue

            if (b'/' in record):
                record = record.split(b'/')[0]

            [keyword, value] = record.split(b'=')
            keyword = str(keyword.strip(), 'ascii')
            value = str(value.strip(), 'ascii')
            self.keywords[keyword] = parseValue(value)

        header_bytes = nkw*80

        # Find the start of the data (a multiple of 2880).
        data_start = 2880
        while (data_start < header_bytes):
            data_start += 2880

        # Check that this is not a color image.
        if ("NAXIS3" in self.keywords):
            raise SimpleFitsException("RGB image detected. Image type not set to 'RAW'?")
            
        # Determine image size.
        if ("NAXIS1" in self.keywords) and ("NAXIS2" in self.keywords):

            size1 = self.keywords["NAXIS1"]
            size2 = self.keywords["NAXIS2"]
            
            # Handle 16bit images.
            if (self.keywords["BITPIX"] == 16):
                if verbose:
                    print(size1, "x", size2, "- 16 bit image")

                # Calculate image size.
                image_size = size1 * size2 * 2

                # Create image.
                self.np_data = numpy.frombuffer(fits_string[data_start:data_start+image_size],
                                                dtype = numpy.dtype('>i2'))
                self.np_data = numpy.reshape(self.np_data, ((size2, size1)))

                return

        raise SimpleFitsException("Unrecognized FITS file type")

    def hasKeyword(self, keyword):
        return keyword in self.keywords
    
    def getKeyword(self, keyword):
        return self.keywords[keyword]
        
    def getKeywords(self):
        return self.keywords

    def getImage(self):
        if ("BZERO" in self.keywords):
            return self.np_data + self.keywords["BZERO"]
        else:
            return self.np_data


if (__name__ == "__main__"):

    import argparse

    parser = argparse.ArgumentParser(description = 'A (simple) FITS format parser')

    parser.add_argument('--fits_file', dest='fits_file', type=str, required=True,
                        help = "The name of the fits file to load.")

    args = parser.parse_args()

    fi = FitsImage(fits_name = args.fits_file)
    print("This file has the following keywords:")
    for key in fi.getKeywords():
        print(" ", key, fi.getKeyword(key))
    print("")

    print("Image size is", fi.getImage().shape)





############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
import logging
from baseclasses.camera import MWCamera
from indi.qt_indi_client import QtINDIClient
import indi.indi_xml as indiXML
import pyfits
import PyQt5


class EKOSCamera(MWCamera):
    logger = logging.getLogger(__name__)                                                                                    # logging enabling

    def __init__(self, app):
        self.app = app
        self.appRunning = False
        self.cameraConnected = False
        self.cameraStatus = ''
        self.appInstallPath = ''
        self.appAvailable = False
        self.appName = 'EKOS - INDI'
        self.appExe = ''
        self.host = '192.168.2.163'
        self.port = 7624
        self.tryConnectionCounter = 0
        self.indiClient = None
        self.indiClient = QtINDIClient()
        self.indiClient.received.connect(self.handleReceived)
        self.checkAppInstall()

    def checkAppInstall(self):
        self.appAvailable = False

    def checkAppStatus(self):
        try:
            print('check App')
            if not self.appRunning:
                PyQt5.QtWidgets.QApplication.processEvents()
                self.indiClient.connect(self.host, self.port)
                if self.indiClient.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                    self.tryConnectionCounter = 0
                    self.appRunning = True
                else:
                    self.tryConnectionCounter += 1
                    self.appRunning = False
                    self.cameraConnected = False
                    if self.tryConnectionCounter < 3:
                        self.logger.warning('{0} is not running'.format(self.appName))
                    elif self.tryConnectionCounter == 3:
                        self.logger.error('No connection to {0} possible - stop logging this connection error'.format(self.appName))
        except Exception as e:
            self.cameraConnected = False
            self.appRunning = False
            self.logger.error('error: {0}'.format(e))
        finally:
            pass

    # central dispatcher for data coming from INDI devices
    def handleReceived(self, message):
        if isinstance(message, indiXML.SetBLOBVector):
            imageHDU = pyfits.HDUList.fromstring(message.getElt(0).getValue())
            imageHeader = imageHDU[0].header
            for key in imageHeader.keys():
                print(key)
        elif isinstance(message, indiXML.SetSwitchVector):
            print(message.getElt(0))
            for elt in message.elt_list:
                if elt.attr['name'] == 'CONNECT':
                    if message.getElt(0).getValue() == 'On':
                        self.cameraConnected = True
                    else:
                        self.cameraConnected = False
        else:
            print(message)

    def send(self, message):
        self.indiClient.sendMessage(message)

    def connectCamera(self):
        if self.appRunning:
            # Connect to the CCD simulator.
            self.indiClient.sendMessage(indiXML.newSwitchVector([indiXML.oneSwitch("On", indi_attr={"name": "CONNECT"})], indi_attr={"name": "CONNECTION", "device": "CCD Simulator"}))
            self.cameraConnected = True

    def disconnectCamera(self):
        if self.cameraConnected:
            # Connect to the CCD simulator.
            self.indiClient.sendMessage(indiXML.newSwitchVector([indiXML.oneSwitch("Off", indi_attr={"name": "CONNECT"})], indi_attr={"name": "CONNECTION", "device": "CCD Simulator"}))
            self.cameraConnected = False

    def getImage(self, modelData):
        if self.cameraConnected:
            # Enable BLOB mode.
            self.indiClient.sendMessage(indiXML.enableBLOB("Also", indi_attr={"device": "CCD Simulator"}))
            self.indiClient.sendMessage(indiXML.newNumberVector([indiXML.oneNumber(2, indi_attr={"name": "CCD_EXPOSURE_VALUE"})], indi_attr={"name": "CCD_EXPOSURE", "device": "CCD Simulator"}))

    def solveImage(self, modelData):
        pass

    def getCameraProps(self):
        if self.cameraConnected:
            pass

    def getCameraStatus(self):
        if self.appRunning:
            self.indiClient.sendMessage(indiXML.clientGetProperties(indi_attr={"version": "1.0", "device": "CCD Simulator"}))
            # Get a list of camera status
            # self.indiClient.sendMessage(indiXML.clientGetProperties(indi_attr={"version": "1.0", "device": "CCD Simulator"}))



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################

import logging
import math
# import basic stuff
import platform
import threading
import time
# for the sorting
from operator import itemgetter

# import PyQT5 for threading purpose
import PyQt5

if platform.system() == 'Windows':
    # win32com
    import pythoncom
#  mount driver classes
if platform.system() == 'Windows':
    from mount import ascommount
from mount import ipdirect
# astrometry
from astrometry import transform


class Mount(PyQt5.QtCore.QThread):
    logger = logging.getLogger(__name__)                                                                                    # enable logging
    signalMountConnected = PyQt5.QtCore.pyqtSignal([bool], name='mountConnected')                                           # signal for connection status
    signalMountAzAltPointer = PyQt5.QtCore.pyqtSignal([float, float], name='mountAzAltPointer')
    signalMountTrackPreview = PyQt5.QtCore.pyqtSignal(name='mountTrackPreview')

    BLUE = 'background-color: rgb(42, 130, 218)'
    DEFAULT = 'background-color: rgb(32,32,32); color: rgb(192,192,192)'
    BLIND_COMMANDS = ['AP', 'hP', 'PO', 'RT0', 'RT1', 'RT2', 'RT9', 'STOP', 'U2']

    def __init__(self, app):
        super().__init__()                                                                                                  # init of the class parent with super
        self.app = app                                                                                                      # accessing ui object from mount class
        if platform.system() == 'Windows':
            self.MountAscom = ascommount.MountAscom(app)                                                                    # set ascom driver class
        self.MountIpDirect = ipdirect.MountIpDirect(app)
        self.mountHandler = self.MountIpDirect
        self.transform = transform.Transform(app)
        self.statusReference = {'0': 'Tracking',
                                '1': 'Stopped after STOP',
                                '2': 'Slewing to park position',
                                '3': 'Unparking',
                                '4': 'Slewing to home position',
                                '5': 'Parked',
                                '6': 'Slewing or going to stop',
                                '7': 'Tracking Off no move',
                                '8': 'Motor low temperature',
                                '9': 'Tracking outside limits',
                                '10': 'Following Satellite',
                                '11': 'User OK Needed',
                                '98': 'Unknown Status',
                                '99': 'Error'
                                }                                                                                           # conversion list Gstat to text
        self.ra = 0                                                                                                         # mount reported ra to J2000 converted
        self.dec = 0                                                                                                        # mount reported dec to J2000 converted
        self.raJnow = 0
        self.decJnow = 0
        self.az = 0                                                                                                         # mount reported azimuth
        self.alt = 0                                                                                                        # mount reported altitude
        self.stat = 0                                                                                                       # mount status (from Gstat command)
        self.slewing = False                                                                                                # from *D' command
        self.site_lat = '49'                                                                                                # site lat
        self.site_lon = '0'                                                                                                 # site lon
        self.site_height = '0'                                                                                              # site height
        self.jd = 2451544.5                                                                                                 # julian date
        self.sidereal_time = ''                                                                                             # local sidereal time
        self.pierside = 0                                                                                                   # side of pier (E/W)
        self.timeToFlip = 200                                                                                               # minutes to flip
        self.timeToMeridian = 0
        self.meridianLimitTrack = 5.0                                                                                       # degrees after meridian to flip
        self.refractionTemp = '20.0'                                                                                        # coordinate transformation need temp
        self.refractionPressure = '900.0'                                                                                   # and pressure
        self.counter = 0                                                                                                    # counter im main loop
        self.chooserLock = threading.Lock()
        self.initConfig()

    def initConfig(self):
        self.app.ui.pd_chooseMountConnection.addItem('IP Direct Connection')
        if platform.system() == 'Windows':
            self.app.ui.pd_chooseMountConnection.addItem('ASCOM Driver Connection')
        try:
            if platform.system() == 'Windows':
                if 'ASCOMTelescopeDriverName' in self.app.config:
                    self.MountAscom.driverName = self.app.config['ASCOMTelescopeDriverName']
            if 'MountConnection' in self.app.config:
                self.app.ui.pd_chooseMountConnection.setCurrentIndex(int(self.app.config['MountConnection']))
                self.showConfigEntries(int(self.app.config['MountConnection']))
            if 'CheckAutoRefractionCamera' in self.app.config:
                self.app.ui.checkAutoRefractionCamera.setChecked(self.app.config['CheckAutoRefractionCamera'])
            if 'CheckAutoRefractionNotTracking' in self.app.config:
                self.app.ui.checkAutoRefractionNotTracking.setChecked(self.app.config['CheckAutoRefractionNotTracking'])

        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass
        self.app.ui.pd_chooseMountConnection.currentIndexChanged.connect(self.chooseMountConn)

    def storeConfig(self):
        if platform.system() == 'Windows':
            self.app.config['ASCOMTelescopeDriverName'] = self.MountAscom.driverName
        self.app.config['MountConnection'] = self.app.ui.pd_chooseMountConnection.currentIndex()
        self.app.config['CheckAutoRefractionCamera'] = self.app.ui.checkAutoRefractionCamera.isChecked()
        self.app.config['CheckAutoRefractionNotTracking'] = self.app.ui.checkAutoRefractionNotTracking.isChecked()

    def showConfigEntries(self, index):
        if index == 0:
            self.app.ui.le_mountIP.setVisible(True)
            self.app.ui.le_mountIP.setEnabled(True)
            self.app.ui.le_mountMAC.setVisible(True)
            self.app.ui.le_mountMAC.setEnabled(True)
            self.app.ui.label_mountIP.setVisible(True)
            self.app.ui.label_mountMAC.setVisible(True)

            self.app.ui.btn_setupMountDriver.setVisible(False)
            self.app.ui.btn_setupMountDriver.setEnabled(False)
        elif index == 1:
            self.app.ui.le_mountIP.setVisible(False)
            self.app.ui.le_mountIP.setEnabled(False)
            self.app.ui.le_mountMAC.setVisible(False)
            self.app.ui.le_mountMAC.setEnabled(False)
            self.app.ui.label_mountIP.setVisible(False)
            self.app.ui.label_mountMAC.setVisible(False)

            self.app.ui.btn_setupMountDriver.setVisible(True)
            self.app.ui.btn_setupMountDriver.setEnabled(True)

    def chooseMountConn(self):
        self.chooserLock.acquire()                                                                                          # avoid multiple switches running at the same time
        if self.mountHandler.connected:
            self.mountHandler.connected = False                                                                             # connection to False -> no commands emitted
            self.mountHandler.disconnect()                                                                                  # do formal disconnection
        if self.app.ui.pd_chooseMountConnection.currentText().startswith('IP Direct Connection'):
            self.mountHandler = self.MountIpDirect
            self.logger.info('actual driver is IpDirect, IP is: {0}'.format(self.MountIpDirect.mountIP()))
        if self.app.ui.pd_chooseMountConnection.currentText().startswith('ASCOM Driver Connection'):
            self.mountHandler = self.MountAscom
            self.logger.info('actual driver is ASCOM')
        self.showConfigEntries(self.app.ui.pd_chooseMountConnection.currentIndex())
        self.chooserLock.release()                                                                                          # free the lock to move again

    def run(self):                                                                                                          # runnable of the thread
        if platform.system() == 'Windows':
            pythoncom.CoInitialize()                                                                                        # needed for doing COM objects in threads
        self.chooseMountConn()
        self.counter = 0                                                                                                    # init count for managing different cycle times
        while True:                                                                                                         # main loop in thread
            self.signalMountConnected.emit(self.mountHandler.connected)                                                     # sending the connection status
            if self.mountHandler.connected:                                                                                 # when connected, starting the work
                if not self.app.mountCommandQueue.empty():                                                                       # checking if in queue is something to do
                    command = self.app.mountCommandQueue.get()                                                                   # if yes, getting the work command
                    if command == 'ShowAlignmentModel':                                                                     # checking which command was sent
                        num = self.numberModelStars()
                        if num == -1:
                            self.app.messageQueue.put('Show Model not available without real mount')
                        else:
                            self.app.mountDataQueue.put({'Name': 'ModelStarError', 'Value': 'delete'})
                            self.app.ui.btn_showActualModel.setStyleSheet(self.BLUE)
                            self.showAlignmentModel(self.getAlignmentModel())
                            self.app.ui.btn_showActualModel.setStyleSheet(self.DEFAULT)
                    elif command == 'ClearAlign':
                        if self.numberModelStars() == -1:
                            self.app.messageQueue.put('Clear Align not available without real mount')
                        else:
                            self.mountHandler.sendCommand('delalig')
                    elif command == 'RunTargetRMSAlignment':
                        if self.numberModelStars() == -1:
                            self.app.messageQueue.put('Run Optimize not available without real mount')
                        else:
                            self.app.ui.btn_runTargetRMSAlignment.setStyleSheet(self.BLUE)
                            self.runTargetRMSAlignment()
                            self.app.ui.btn_runTargetRMSAlignment.setStyleSheet(self.DEFAULT)
                    elif command == 'DeleteWorstPoint':
                        if self.numberModelStars() == -1:
                            self.app.messageQueue.put('Delete worst point not available without real mount')
                        else:
                            self.app.ui.btn_deleteWorstPoint.setStyleSheet(self.BLUE)
                            self.deleteWorstPoint()
                            self.app.ui.btn_deleteWorstPoint.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveBackupModel':
                        self.app.ui.btn_saveBackupModel.setStyleSheet(self.BLUE)                                                # button blue
                        self.saveBackupModel()
                        self.app.ui.btn_saveBackupModel.setStyleSheet(self.DEFAULT)                                             # button to default back
                    elif command == 'LoadBackupModel':
                        self.app.ui.btn_loadBackupModel.setStyleSheet(self.BLUE)
                        self.loadBackupModel()
                        self.app.ui.btn_loadBackupModel.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadBaseModel':
                        self.app.ui.btn_loadBaseModel.setStyleSheet(self.BLUE)
                        self.loadBaseModel()
                        self.app.ui.btn_loadBaseModel.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveBaseModel':
                        self.app.ui.btn_saveBaseModel.setStyleSheet(self.BLUE)
                        self.saveBaseModel()
                        self.app.ui.btn_saveBaseModel.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadRefinementModel':
                        self.app.ui.btn_loadRefinementModel.setStyleSheet(self.BLUE)
                        self.loadRefinementModel()
                        self.app.ui.btn_loadRefinementModel.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveRefinementModel':
                        self.app.ui.btn_saveRefinementModel.setStyleSheet(self.BLUE)
                        self.saveRefinementModel()
                        self.app.ui.btn_saveRefinementModel.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadSimpleModel':
                        self.app.ui.btn_loadSimpleModel.setStyleSheet(self.BLUE)
                        self.loadSimpleModel()
                        self.app.ui.btn_loadSimpleModel.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveSimpleModel':
                        self.app.ui.btn_saveSimpleModel.setStyleSheet(self.BLUE)
                        self.saveSimpleModel()
                        self.app.ui.btn_saveSimpleModel.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadDSO1Model':
                        self.app.ui.btn_loadDSO1Model.setStyleSheet(self.BLUE)
                        self.loadDSO1Model()
                        self.app.ui.btn_loadDSO1Model.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveDSO1Model':
                        self.app.ui.btn_saveDSO1Model.setStyleSheet(self.BLUE)
                        self.saveDSO1Model()
                        self.app.ui.btn_saveDSO1Model.setStyleSheet(self.DEFAULT)
                    elif command == 'LoadDSO2Model':
                        self.app.ui.btn_loadDSO2Model.setStyleSheet(self.BLUE)
                        self.loadDSO2Model()
                        self.app.ui.btn_loadDSO2Model.setStyleSheet(self.DEFAULT)
                    elif command == 'SaveDSO2Model':
                        self.app.ui.btn_saveDSO2Model.setStyleSheet(self.BLUE)
                        self.saveDSO2Model()
                        self.app.ui.btn_saveDSO2Model.setStyleSheet(self.DEFAULT)
                    elif command == 'SetRefractionParameter':
                        self.setRefractionParameter()
                    elif command == 'FLIP':
                        self.flipMount()
                    elif command == 'SetupAscomDriver':
                        self.MountAscom.setupDriver()
                    elif command == 'Shutdown':
                        self.mountShutdown()
                    else:
                        self.mountHandler.sendCommand(command)                                                              # doing the command directly to mount (no method necessary)
                    self.app.mountCommandQueue.task_done()
                else:                                                                                                       # if not connected, the we should do this
                    if self.counter == 0:                                                                                   # jobs once done at the beginning
                        self.getStatusOnce()                                                                                # task once
                    if self.counter % 2 == 0:                                                                               # all tasks with 400 ms
                        self.getStatusFast()                                                                                # polling the mount status Ginfo
                    if self.counter % 15 == 0:                                                                              # all tasks with 3 s
                        self.getStatusMedium()                                                                              # polling the mount
                    if self.counter % 150 == 0:                                                                             # all task with 30 seconds
                        self.getStatusSlow()                                                                                # slow ones
                time.sleep(0.2)                                                                                             # time base is 200 ms
                self.counter += 1                                                                                           # increasing counter for selection
            else:                                                                                                           # when not connected try to connect
                self.mountHandler.connect()
                self.counter = 0
                time.sleep(1)                                                                                               # try it every second, not more
        self.mountHandler.disconnect()
        if platform.system() == 'Windows':
            pythoncom.CoUninitialize()                                                                                      # needed for doing COM objects in threads
        self.terminate()                                                                                                    # closing the thread at the end

    def __del__(self):                                                                                                      # remove thread
        self.wait()                                                                                                         # wait for stop of thread

    def mountShutdown(self):
        reply = self.mountHandler.sendCommand('shutdown')
        if reply != '1':
            self.logger.error('error: {0}'.format(reply))
            self.app.messageQueue.put('Error in mount shutdown !')
        else:
            self.mountHandler.connected = False                                                                             # connection to False -> no commands emitted
            time.sleep(1)
            self.mountHandler.disconnect()
            self.logger.info('Shutdown mount manually')
            self.app.messageQueue.put('Shutting mount down !')

    def flipMount(self):                                                                                                    # doing the flip of the mount
        reply = self.mountHandler.sendCommand('FLIP').rstrip('#').strip()
        if reply == '0':                                                                                                    # error handling if not successful
            self.app.messageQueue.put('Flip Mount could not be executed !')                                                 # write to gui
            self.logger.error('error: {0}'.format(reply))

    def numberModelStars(self):
        return int(self.mountHandler.sendCommand('getalst'))                                                                # if there are some points, a modeling must be there

    def getAlignmentModelStatus(self, alignModel):
        try:
            reply = self.mountHandler.sendCommand('getain')                                                                 # load the data from new command
            if reply:                                                                                                       # there should be a reply, format string is "ZZZ.ZZZZ,+AA.AAAA,EE.EEEE,PPP.PP,+OO.OOOO,+aa.aa, +bb.bb,NN,RRRRR.R#"
                if reply != 'E':                                                                                            # if a single 'E' returns, there is a problem, not further parameter will follow
                    a1, a2, a3, a4, a5, a6, a7, a8, a9 = reply.split(',')
                    if a1 != 'E':                                                                                           # 'E' could be sent if not calculable or no value available
                        alignModel['azimuth'] = float(a1)
                    else:
                        alignModel['azimuth'] = 0
                    if a2 != 'E':
                        alignModel['altitude'] = float(a2)
                    else:
                        alignModel['altitude'] = 0
                    if a3 != 'E':
                        alignModel['polarError'] = float(a3)
                    else:
                        alignModel['polarError'] = 0
                    if a4 != 'E':
                        alignModel['posAngle'] = float(a4)
                    else:
                        alignModel['posAngle'] = 0
                    if a5 != 'E':
                        alignModel['orthoError'] = float(a5)
                    else:
                        alignModel['orthoError'] = 0
                    if a6 != 'E':
                        alignModel['azimuthKnobs'] = float(a6)
                    else:
                        alignModel['azimuthKnobs'] = 0
                    if a7 != 'E':
                        alignModel['altitudeKnobs'] = float(a7)
                    else:
                        alignModel['altitudeKnobs'] = 0
                    if a8 != 'E':
                        alignModel['terms'] = int(float(a8))
                    else:
                        alignModel['terms'] = 0
                    if a9 != 'E':
                        alignModel['RMS'] = float(a9)
                    else:
                        alignModel['RMS'] = 0
        except Exception as e:
            self.logger.error('receive error getain command: {0}'.format(e))
        finally:
            return alignModel

    def getAlignmentModel(self):                                                                                            # download alignment modeling from mount
        alignModel = {}                                                                                                     # clear alignment data
        points = []                                                                                                         # clear points list
        alignModel['azimuth'] = 0.0
        alignModel['altitude'] = 0.0
        alignModel['polarError'] = 0.0
        alignModel['posAngle'] = 0.0
        alignModel['orthoError'] = 0.0
        alignModel['azimuthKnobs'] = 0.0
        alignModel['altitudeKnobs'] = 0.0
        alignModel['terms'] = 0
        alignModel['RMS'] = 0.0                                                                                             # clear the alignment points downloaded
        alignModel['points'] = points
        numberStars = self.numberModelStars()                                                                               # get number of stars
        alignModel['number'] = numberStars
        if numberStars < 1:                                                                                                 # if no stars or no real mount, finish
            return alignModel
        alignModel = self.getAlignmentModelStatus(alignModel)                                                               # add status information
        for i in range(1, numberStars + 1):                                                                                 # otherwise download them step for step
            reply = self.mountHandler.sendCommand('getalp{0:d}'.format(i)).split(',')
            ha = reply[0].strip().split('.')[0]
            dec = reply[1].strip().split('.')[0]
            errorRMS = float(reply[2].strip())
            errorAngle = reply[3].strip().rstrip('#')
            dec = dec.replace('*', ':')
            ra_J2000 = self.transform.degStringToDecimal(ha)
            dec_J2000 = self.transform.degStringToDecimal(dec)
            az, alt = self.transform.ra_dec_lst_to_az_alt(ra_J2000, dec_J2000)
            alignModel['points'].append((i-1, ra_J2000, dec_J2000, az, alt, errorRMS, float(errorAngle)))                   # index should start with 0, but numbering in mount starts with 1
        return alignModel

    def retrofitMountData(self, data):
        num = self.numberModelStars()                                                                                       # size mount modeling
        if num == len(data):
            alignModel = self.getAlignmentModel()                                                                           # get mount points
            self.showAlignmentModel(alignModel)
            for i in range(0, alignModel['number']):                                                                        # run through all the points
                data[i]['modelError'] = float(alignModel['points'][i][5])                                                   # and for the total error
                data[i]['raError'] = data[i]['modelError'] * math.sin(math.radians(alignModel['points'][i][6]))             # set raError new from total error mount with polar error angle from mount
                data[i]['decError'] = data[i]['modelError'] * math.cos(math.radians(alignModel['points'][i][6]))            # same to dec
            self.app.modelLogQueue.put('Mount Model and Model Data synced\n')
        else:
            self.logger.warning('size mount modeling {0} and modeling data {1} do not fit !'.format(num, len(data)))
            self.app.modelLogQueue.put('Mount Model and Model Data could not be synced\n')
            self.app.messageQueue.put('Error- Mount Model and Model Data mismatch!\n')
        return data

    def showAlignmentModel(self, alignModel):
        self.app.mountDataQueue.put({'Name': 'ModelStarError', 'Value': 'Downloading data\n'})
        for i in range(0, alignModel['number']):
            self.app.mountDataQueue.put({'Name': 'ModelStarError', 'Value': '#{0:02d}   AZ: {1:3d}   Alt: {2:3d}   Err: {3:4.1f}\x22   PA: {4:3.0f}\xb0\n'
                                        .format(i, int(alignModel['points'][i][3]), int(alignModel['points'][i][4]), alignModel['points'][i][5], alignModel['points'][i][6])})
        self.app.mountDataQueue.put({'Name': 'ModelStarError', 'Value': 'Downloading finished\n'})
        self.app.mountDataQueue.put({'Name': 'NumberAlignmentStars', 'Value': alignModel['number']})
        self.app.mountDataQueue.put({'Name': 'ModelRMSError', 'Value': '{0:3.1f}'.format(alignModel['RMS'])})
        self.app.mountDataQueue.put({'Name': 'ModelRMSError', 'Value': '{0:3.1f}'.format(alignModel['RMS'])})
        self.app.mountDataQueue.put({'Name': 'ModelErrorPosAngle', 'Value': '{0:3.1f}'.format(alignModel['posAngle'])})
        self.app.mountDataQueue.put({'Name': 'ModelPolarError', 'Value': '{0}'.format(self.transform.decimalToDegree(alignModel['polarError']))})
        self.app.mountDataQueue.put({'Name': 'ModelOrthoError', 'Value': '{0}'.format(self.transform.decimalToDegree(alignModel['orthoError']))})
        self.app.mountDataQueue.put({'Name': 'ModelErrorAz', 'Value': '{0}'.format(self.transform.decimalToDegree(alignModel['azimuthKnobs']))})
        self.app.mountDataQueue.put({'Name': 'ModelErrorAlt', 'Value': '{0}'.format(self.transform.decimalToDegree(alignModel['altitudeKnobs']))})
        self.app.mountDataQueue.put({'Name': 'ModelTerms', 'Value': '{0:2d}'.format(alignModel['terms'])})
        if alignModel['azimuthKnobs'] > 0:
            value = '{0:2.2f} left'.format(abs(alignModel['azimuthKnobs']))
        else:
            value = '{0:2.2f} right'.format(abs(alignModel['azimuthKnobs']))
        self.app.mountDataQueue.put({'Name': 'ModelKnobTurnAz', 'Value': '{0}'.format(value)})
        if alignModel['altitudeKnobs'] > 0:
            value = '{0:2.2f} down'.format(abs(alignModel['altitudeKnobs']))
        else:
            value = '{0:2.2f} up'.format(abs(alignModel['altitudeKnobs']))
        self.app.mountDataQueue.put({'Name': 'ModelKnobTurnAlt', 'Value': '{0}'.format(value)})
        self.app.showModelErrorPolar()
        return

    def runTargetRMSAlignment(self):
        self.app.mountDataQueue.put({'Name': 'ModelStarError', 'Value': 'delete'})
        alignModel = self.getAlignmentModel()
        if alignModel['number'] < 4:
            return                                                                                                          # set maximum
        while alignModel['RMS'] > float(self.app.ui.targetRMS.value()) and alignModel['number'] > 3:
            alignModel = self.deleteWorstPointRaw(alignModel)

    def deleteWorstPoint(self):
        alignModel = self.getAlignmentModel()
        self.deleteWorstPointRaw(alignModel)

    def deleteWorstPointRaw(self, alignModel):
        if alignModel['number'] < 4:
            return
        if alignModel['number'] > 3:
            a = sorted(alignModel['points'], key=itemgetter(5), reverse=True)                                               # index 0 is the worst star, index starts with 0
            index = a[0][0]
            reply = self.mountHandler.sendCommand('delalst{0:d}'.format(index + 1))                                         # numbering in mount starts with 1
            if reply == '1':                                                                                                # worst point could be deleted
                alignModel = self.getAlignmentModel()
                self.app.modeling.modelData.pop(index)
                for i in range(0, alignModel['number']):
                    self.app.modeling.modelData[i]['modelError'] = float(alignModel['points'][i][5])
                    self.app.modeling.modelData[i]['raError'] = self.app.modeling.modelData[i]['modelError'] * math.sin(math.radians(float(alignModel['points'][i][6])))
                    self.app.modeling.modelData[i]['decError'] = self.app.modeling.modelData[i]['modelError'] * math.cos(math.radians(float(alignModel['points'][i][6])))
                self.showAlignmentModel(alignModel)
            else:
                self.logger.warning('Point {0} could not be deleted').format(index)
        return alignModel

    def saveModel(self, target):
        num = self.numberModelStars()
        if num == -1:
            self.app.messageQueue.put('Save Model not available without real mount')
            return False
        self.mountHandler.sendCommand('modeldel0' + target)
        reply = self.mountHandler.sendCommand('modelsv0' + target)
        if reply == '1':
            self.app.messageQueue.put('Actual Mount Model saved to file {0}'.format(target))
            return True
        else:
            self.logger.warning('Model {0} could not be saved'.format(target))
            return False

    def loadModel(self, target):
        num = self.numberModelStars()
        if num == -1:
            self.app.messageQueue.put('Load Model not available without real mount')
            return False
        reply = self.mountHandler.sendCommand('modelld0' + target)
        if reply == '1':
            self.app.messageQueue.put('Mount Model loaded from file {0}'.format(target))
            return True
        else:
            self.app.messageQueue.put('There is no modeling named {0} or error while loading'.format(target))
            self.logger.warning('Model {0} could not be loaded'.format(target))
            return False

    def saveBackupModel(self):
        if self.saveModel('BACKUP'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'backup.dat')

    def loadBackupModel(self):
        if self.loadModel('BACKUP'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('backup.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for BACKUP')

    def saveBaseModel(self):
        if self.saveModel('BASE'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'base.dat')
            else:
                self.app.messageQueue.put('No data for BASE')

    def loadBaseModel(self):
        if self.loadModel('BASE'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('base.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for BASE')

    def saveRefinementModel(self):
        if self.saveModel('REFINE'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'refine.dat')
            else:
                self.app.messageQueue.put('No data for REFINE')

    def loadRefinementModel(self):
        if self.loadModel('REFINE'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('refine.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for REFINE')

    def saveActualModel(self):
        if self.saveModel('ACTUAL'):
            if self.app.modeling.modelData:
                if 'index' in self.app.modeling.modelData[0].keys():
                    self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'actual.dat')
            else:
                self.app.messageQueue.put('No data for ACTUAL')

    def loadActualModel(self):
        if self.loadModel('ACTUAL'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('actual.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for ACTUAL')

    def saveSimpleModel(self):
        if self.saveModel('SIMPLE'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'simple.dat')
            else:
                self.app.messageQueue.put('No data file for SIMPLE')

    def loadSimpleModel(self):
        if self.loadModel('SIMPLE'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('simple.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for SIMPLE')

    def saveDSO1Model(self):
        if self.saveModel('DSO1'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'DSO1.dat')
            else:
                self.app.messageQueue.put('No data file for DSO1')

    def loadDSO1Model(self):
        if self.loadModel('DSO1'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('DSO1.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for DSO1')

    def saveDSO2Model(self):
        if self.saveModel('DSO2'):
            if self.app.modeling.modelData:
                self.app.analyseWindow.analyse.saveData(self.app.modeling.modelData, 'DSO2.dat')
            else:
                self.app.messageQueue.put('No data file for DSO2')

    def loadDSO2Model(self):
        if self.loadModel('DSO2'):
            self.app.modeling.modelData = self.app.analyseWindow.analyse.loadDataRaw('dso2.dat')
            if not self.app.modeling.modelData:
                self.app.messageQueue.put('No data file for DSO2')

    def setRefractionParameter(self):
        if self.app.ui.le_pressure.text() != '---' and self.app.ui.le_temperature.text() != '---':
            pressure = float(self.app.ui.le_pressure.text() + '0')
            temperature = float(self.app.ui.le_temperature.text() + '0')
            if self.app.environment.connected == 1:
                if (900.0 < pressure < 1100.0) and (-40.0 < temperature < 50.0):
                    self.mountHandler.sendCommand('SRPRS{0:04.1f}'.format(float(self.app.ui.le_pressure.text())))
                    if temperature > 0:
                        self.mountHandler.sendCommand('SRTMP+{0:03.1f}'.format(float(self.app.ui.le_temperature.text())))
                    else:
                        self.mountHandler.sendCommand('SRTMP-{0:3.1f}'.format(-float(self.app.ui.le_temperature.text())))
                    self.app.mountDataQueue.put({'Name': 'GetRefractionTemperature', 'Value': self.mountHandler.sendCommand('GRTMP')})
                    self.app.mountDataQueue.put({'Name': 'GetRefractionPressure', 'Value': self.mountHandler.sendCommand('GRPRS')})
                else:
                    # todo : better check before writing warning
                    self.logger.warning('parameters out of range ! temperature:{0} pressure:{1}'.format(temperature, pressure))

    def getStatusFast(self):                                                                                                # fast status item like pointing
        reply = self.mountHandler.sendCommand('GS')
        if reply:
            self.sidereal_time = reply.strip('#')
            self.app.mountDataQueue.put({'Name': 'GetLocalSiderealTime', 'Value': '{0}'.format(self.sidereal_time)})        # Sidereal local time
        reply = self.mountHandler.sendCommand('GR')
        if reply:
            self.raJnow = self.transform.degStringToDecimal(reply)
        reply = self.mountHandler.sendCommand('GD')
        if reply:
            self.decJnow = self.transform.degStringToDecimal(reply)
        reply = self.mountHandler.sendCommand('Ginfo')                                                                      # use command "Ginfo" for fast topics
        if reply:
            ra = ''
            dec = ''
            jd = ''
            az = ''
            alt = ''
            stat = ''
            slew = ''
            try:
                ra, dec, self.pierside, az, alt, jd, stat, slew = reply.rstrip('#').strip().split(',')                      # split the response to its parts
            except Exception as e:
                self.logger.error('receive error Ginfo command: {0} reply:{1}'.format(e, reply))
            finally:
                pass
            self.raJnow = float(ra)
            self.decJnow = float(dec)
            self.jd = jd.rstrip('#')                                                                                        # needed for 2.14.8 beta firmware
            self.az = float(az)                                                                                             # same to azimuth
            self.alt = float(alt)                                                                                           # and altitude
            self.stat = int(stat)                                                                                           # status should be int for referencing list
            self.slewing = (slew == '1')                                                                                    # set status slewing
            self.ra, self.dec = self.transform.transformERFA(self.raJnow, self.decJnow, 2)                                  # convert J2000
            ra_show = self.transform.decimalToDegree(self.ra, False, False)
            dec_show = self.transform.decimalToDegree(self.dec, True, False)
            self.app.mountDataQueue.put({'Name': 'GetTelescopeDEC', 'Value': '{0}'.format(dec_show)})                       # put dec to gui
            self.app.mountDataQueue.put({'Name': 'GetTelescopeRA', 'Value': '{0}'.format(ra_show)})                         # put ra to gui
            self.app.mountDataQueue.put({'Name': 'GetTelescopeAltitude', 'Value': '{0:03.2f}'.format(self.alt)})            # Altitude
            self.app.mountDataQueue.put({'Name': 'GetTelescopeAzimuth', 'Value': '{0:03.2f}'.format(self.az)})              # Azimuth
            self.app.mountDataQueue.put({'Name': 'GetMountStatus', 'Value': '{0}'.format(self.stat)})                       # Mount status -> slew to stop
            self.app.mountDataQueue.put({'Name': 'GetJulianDate', 'Value': '{0}'.format(self.jd[:13])})                     # Sidereal local time
            if str(self.pierside) == str('W'):                                                                              # pier side
                self.app.mountDataQueue.put({'Name': 'GetTelescopePierSide', 'Value': 'WEST'})                              # Transfer to test in GUI
            else:                                                                                                           #
                self.app.mountDataQueue.put({'Name': 'GetTelescopePierSide', 'Value': 'EAST'})                              # Transfer to Text for GUI
            self.signalMountAzAltPointer.emit(self.az, self.alt)                                                            # set azalt Pointer in diagrams to actual pos
            self.timeToFlip = int(float(self.mountHandler.sendCommand('Gmte')))
            self.meridianLimitTrack = int(float(self.mountHandler.sendCommand('Glmt')))
            self.timeToMeridian = int(self.timeToFlip - self.meridianLimitTrack / 360 * 24 * 60)
            self.app.mountDataQueue.put({'Name': 'GetMeridianLimitTrack', 'Value': self.meridianLimitTrack})
            self.app.mountDataQueue.put({'Name': 'GetMeridianLimitSlew', 'Value': int(float(self.mountHandler.sendCommand('Glms')))})
            self.app.mountDataQueue.put({'Name': 'GetTimeToFlip', 'Value': self.timeToFlip})                                # Flip time
            self.app.mountDataQueue.put({'Name': 'GetTimeToMeridian', 'Value': self.timeToMeridian})                        # Time to meridian

    def getStatusMedium(self):                                                                                              # medium status items like refraction
        if self.app.ui.checkAutoRefractionNotTracking.isChecked():
            if self.stat != 0:                                                                                              # if no tracking, than autorefraction is good
                self.setRefractionParameter()
        if self.app.ui.checkAutoRefractionCamera.isChecked():                                                               # check if autorefraction is set
            if self.app.modeling.imagingHandler.cameraStatus in ['READY - IDLE', 'DOWNLOADING']:                            # if tracking, when camera is idle or downloading
                self.setRefractionParameter()                                                                               # transfer refraction to mount
        self.app.mountDataQueue.put({'Name': 'GetSlewRate', 'Value': self.mountHandler.sendCommand('GMs')})                 # get actual slew rate
        self.signalMountTrackPreview.emit()

    def getStatusSlow(self):                                                                                                # slow update item like temps
        self.timeToFlip = self.mountHandler.sendCommand('Gmte')
        self.app.mountDataQueue.put({'Name': 'GetTimeToTrackingLimit', 'Value': self.timeToFlip})                           # Flip time
        self.refractionTemp = self.mountHandler.sendCommand('GRTMP')
        self.app.mountDataQueue.put({'Name': 'GetRefractionTemperature', 'Value': self.refractionTemp})                     # refraction temp out of mount
        self.refractionPressure = self.mountHandler.sendCommand('GRPRS')
        self.app.mountDataQueue.put({'Name': 'GetRefractionPressure', 'Value': self.refractionPressure})                    # refraction pressure out of mount
        self.app.mountDataQueue.put({'Name': 'GetTelescopeTempDEC', 'Value': self.mountHandler.sendCommand('GTMP1')})       # temp motor circuit of both axes
        self.app.mountDataQueue.put({'Name': 'GetRefractionStatus', 'Value': self.mountHandler.sendCommand('GREF')})
        self.app.mountDataQueue.put({'Name': 'GetUnattendedFlip', 'Value': self.mountHandler.sendCommand('Guaf')})
        self.app.mountDataQueue.put({'Name': 'GetMeridianLimitTrack', 'Value': self.mountHandler.sendCommand('Glmt')})
        self.app.mountDataQueue.put({'Name': 'GetMeridianLimitSlew', 'Value': self.mountHandler.sendCommand('Glms')})
        self.app.mountDataQueue.put({'Name': 'GetDualAxisTracking', 'Value': self.mountHandler.sendCommand('Gdat')})
        self.app.mountDataQueue.put({'Name': 'GetCurrentHorizonLimitHigh', 'Value': self.mountHandler.sendCommand('Gh')})
        self.app.mountDataQueue.put({'Name': 'GetCurrentHorizonLimitLow', 'Value': self.mountHandler.sendCommand('Go')})
        try:
            reply = self.mountHandler.sendCommand('GDUTV')
            if reply:
                valid, expirationDate = reply.split(',')
                self.app.mountDataQueue.put({'Name': 'GetUTCDataValid', 'Value': valid})
                self.app.mountDataQueue.put({'Name': 'GetUTCDataExpirationDate', 'Value': expirationDate})
        except Exception as e:
            self.logger.error('receive error GDUTV command: {0}'.format(e))
        finally:
            pass

    def getStatusOnce(self):                                                                                                # one time updates for settings
        self.mountHandler.sendCommand('U2')                                                                                 # Set high precision mode
        self.site_height = self.mountHandler.sendCommand('Gev')                                                             # site height
        lon1 = self.mountHandler.sendCommand('Gg')                                                                          # get site lon
        if lon1[0] == '-':                                                                                                  # due to compatibility to LX200 protocol east is negative
            self.site_lon = lon1.replace('-', '+')                                                                          # change that
        else:
            self.site_lon = lon1.replace('+', '-')                                                                          # and vice versa
        self.site_lat = self.mountHandler.sendCommand('Gt')                                                                 # get site latitude
        self.app.mountDataQueue.put({'Name': 'GetCurrentSiteElevation', 'Value': self.site_height})                         # write data to GUI
        self.app.mountDataQueue.put({'Name': 'GetCurrentSiteLongitude', 'Value': lon1})
        self.app.mountDataQueue.put({'Name': 'GetCurrentSiteLatitude', 'Value': self.site_lat})
        self.app.mountDataQueue.put({'Name': 'GetFirmwareDate', 'Value': self.mountHandler.sendCommand('GVD')})
        self.app.mountDataQueue.put({'Name': 'GetFirmwareNumber', 'Value': self.mountHandler.sendCommand('GVN')})
        self.app.mountDataQueue.put({'Name': 'GetFirmwareProductName', 'Value': self.mountHandler.sendCommand('GVP')})
        self.app.mountDataQueue.put({'Name': 'GetFirmwareTime', 'Value': self.mountHandler.sendCommand('GVT')})
        self.app.mountDataQueue.put({'Name': 'GetHardwareVersion', 'Value': self.mountHandler.sendCommand('GVZ')})
        self.logger.info('FW:{0}'.format(self.mountHandler.sendCommand('GVN')))                                             # firmware version for checking
        self.logger.info('Site Lon:{0}'.format(self.site_lon))                                                              # site lon
        self.logger.info('Site Lat:{0}'.format(self.site_lat))                                                              # site lat
        self.logger.info('Site Height:{0}'.format(self.site_height))                                                        # site height
        self.loadActualModel()                                                                                              # prepare data synchronisation, load modeling data
        alignModel = self.getAlignmentModel()                                                                               # get modeling data from mount
        if not self.app.modeling.modelData and alignModel['RMS'] > 0:
            self.app.messageQueue.put('Model Data will be reconstructed from Mount Data')
            self.app.modeling.modelData = []
            for i in range(0, alignModel['number']):
                self.app.modeling.modelData.append({'modelError': float(alignModel['points'][i][5]),
                                                    'raError': float(alignModel['points'][i][5]) * math.sin(math.radians(alignModel['points'][i][6])),
                                                    'decError': float(alignModel['points'][i][5]) * math.cos(math.radians(alignModel['points'][i][6])),
                                                    'azimuth': float(alignModel['points'][i][3]),
                                                    'altitude': float(alignModel['points'][i][4])})
        self.showAlignmentModel(alignModel)



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
import platform
import logging
import PyQt5
import time
if platform.system() == 'Windows':
    from win32com.client.dynamic import Dispatch
    import pythoncom


class Environment(PyQt5.QtCore.QThread):
    # signals for communication to main Thread / GUI
    logger = logging.getLogger(__name__)
    signalEnvironmentConnected = PyQt5.QtCore.pyqtSignal([int], name='environmentConnected')

    def __init__(self, app):
        super().__init__()
        self.app = app
        self.connected = 2
        self.ascom = None                                                                                                   # placeholder for ascom driver object
        self.chooser = None                                                                                                 # placeholder for ascom chooser object
        self.driverName = ''                                                                                                # driver object name
        self.counter = 0
        self.initConfig()

    def initConfig(self):
        if platform.system() == 'Windows':
            try:
                if 'EnvironmentDriverName' in self.app.config:
                    self.driverName = self.app.config['EnvironmentDriverName']
            except Exception as e:
                self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
            finally:
                pass

    def storeConfig(self):
        self.app.config['EnvironmentDriverName'] = self.driverName

    def run(self):                                                                                                          # runnable for doing the work
        if platform.system() == 'Windows':
            pythoncom.CoInitialize()                                                                                        # needed for doing CO objects in threads
        self.connected = 0                                                                                                  # set connection flag for stick itself
        self.counter = 0
        while True:                                                                                                         # main loop for stick thread
            self.signalEnvironmentConnected.emit(self.connected)                                                            # send status to GUI
            if self.connected == 1:                                                                                         # differentiate between dome connected or not
                if self.counter == 0:                                                                                       # jobs once done at the beginning
                    self.getStatusOnce()                                                                                    # task once
                if self.counter % 2 == 0:                                                                                   # all tasks with 200 ms
                    self.getStatusFast()                                                                                    # polling the mount status Ginfo
                if self.counter % 20 == 0:                                                                                  # all tasks with 3 s
                    self.getStatusMedium()                                                                                  # polling the mount
                if self.counter % 300 == 0:                                                                                 # all task with 1 minute
                    self.getStatusSlow()                                                                                    # slow ones
                self.counter += 1                                                                                           # increasing counter for selection
                time.sleep(.1)
            else:
                try:
                    if self.driverName == '':
                        self.connected = 2
                    else:
                        if platform.system() == 'Windows':
                            self.ascom = Dispatch(self.driverName)                                                          # load driver
                            self.ascom.connected = True
                        self.connected = 1                                                                                  # set status to connected
                        self.logger.info('driver chosen:{0}'.format(self.driverName))
                except Exception as e:                                                                                      # if general except Exception
                    if self.driverName != '':
                        self.logger.error('general error: {0}'.format(e))
                    if self.driverName == '':
                        self.connected = 2
                    else:
                        self.connected = 0                                                                                  # run the driver setup dialog
                finally:                                                                                                    # still continua and try it again
                    pass                                                                                                    # needed for continue
                time.sleep(5)                                                                                               # wait for the next cycle
        if platform.system() == 'Windows':
            self.ascom.Quit()
            pythoncom.CoUninitialize()                                                                                      # needed for doing COm objects in threads
        self.terminate()                                                                                                    # closing the thread at the end

    def __del__(self):                                                                                                      # remove thread
        self.wait()                                                                                                         #

    def getStatusFast(self):
        pass

    def getStatusMedium(self):
        data = dict()
        try:
            data['DewPoint'] = self.ascom.DewPoint                                                                          # storing data in the signal object
        except Exception:
            pass
        finally:
            pass
        try:
            data['Temperature'] = self.ascom.Temperature                                                                    # actually there is single based communication
        except Exception:
            pass
        finally:
            pass
        try:
            data['Humidity'] = self.ascom.Humidity                                                                          # target should be queue
        except Exception:
            pass
        finally:
            pass
        try:
            data['Pressure'] = self.ascom.Pressure
        except Exception:
            pass
        finally:
            pass
        try:
            data['SQR'] = self.ascom.SkyQuality                                                                             # storing data in the signal object
        except Exception:
            pass
        finally:
            pass
        try:
            data['CloudCover'] = self.ascom.CloudCover
        except Exception:
            pass
        finally:
            pass
        try:
            data['RainRate'] = self.ascom.RainRate
        except Exception:
            pass
        finally:
            pass
        try:
            data['WindSpeed'] = self.ascom.WindSpeed
        except Exception:
            pass
        finally:
            pass
        try:
            data['WindDirection'] = self.ascom.WindDirection
        except Exception:
            pass
        finally:
            pass
        self.app.environmentQueue.put(data)                                                                                 # sending the data via signal

    def getStatusSlow(self):
        pass

    def getStatusOnce(self):
        pass

    def setupDriver(self):
        try:
            if platform.system() == 'Windows':
                if self.ascom:
                    self.ascom.connected = False
                    self.ascom = None
                self.chooser = Dispatch('ASCOM.Utilities.Chooser')
                self.chooser.DeviceType = 'ObservingConditions'
                self.driverName = self.chooser.Choose(self.driverName)
                self.logger.info('driver chosen:{0}'.format(self.driverName))
                if self.driverName == '':
                    self.connected = 2
                else:
                    self.connected = 0
        except Exception as e:
            self.app.messageQueue.put('Driver error as in setupEnvironment')
            self.logger.error('general error:{0}'.format(e))
            if self.driverName == '':
                self.connected = 2
            else:
                self.connected = 0                                                                                          # run the driver setup dialog
        finally:                                                                                                            # continue to work
            pass                                                                                                            # python necessary



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
import logging
import os
# for the sorting
import operator


class ModelPoints:
    logger = logging.getLogger(__name__)                                                                                    # logging enabling

    def __init__(self, transform):
        self.transform = transform
        self.horizonPoints = []                                                                                             # point out of file for showing the horizon
        self.BasePoints = []                                                                                                # base point out of a file for modeling
        self.RefinementPoints = []                                                                                          # refinement point out of file for modeling

    def loadModelPoints(self, modelPointsFileName, modeltype):                                                              # load modeling point file from MM als list from tuples
        p = []
        number = 0
        msg = None
        if modelPointsFileName.strip() == '':
            msg = 'No Model Points Filename given!'
            self.logger.warning('No Model Points Filename given!')
            return p, msg
        try:                                                                                                                # fault tolerance, if file io fails
            with open('config/' + modelPointsFileName, 'r') as fileHandle:                                                  # run over complete file
                for line in fileHandle:                                                                                     # run over lines
                    if line.startswith('GRID'):                                                                             # if grid, then its a TSX file
                        convertedLine = line.rstrip('\n').split()                                                           # format is TSX format
                        point = (float(convertedLine[2]), float(convertedLine[3]))                                          # take data from line
                        number += 1
                        if modeltype == 'refinement' and number > 3:                                                        # in MM format base and refinement are included
                            p.append(point)                                                                                 # add data to the adequate list
                        elif modeltype == 'base' and number <= 3:
                            p.append(point)                                                                                 # add data to the adequate list
                    else:
                        convertedLine = line.rstrip('\n').split(':')                                                        # format is same as Per's MM
                        point = (int(convertedLine[0]), int(convertedLine[1]))                                              # take data from line
                        if len(convertedLine) == 2 and modeltype == 'refinement':                                           # in MM format base and refinement are included
                            p.append(point)                                                                                 # add data to the adequate list
                        elif len(convertedLine) != 2 and modeltype == 'base':
                            p.append(point)                                                                                 # close file
        except Exception as e:                                                                                              # handle exception
            msg = 'Error loading modeling points from file [{0}] error: {1}!'.format(modelPointsFileName, e)
            self.logger.warning('Error loading modeling points from file [{0}] error: {1}!'.format(modelPointsFileName, e))
        finally:
            return p, msg

    def sortPoints(self, modeltype):                                                                                        # sorting point for performance
        if modeltype == 'base':                                                                                             # check type of sorting
            points = self.BasePoints                                                                                        # starting with point equals base points
        else:                                                                                                               # otherwise
            points = self.RefinementPoints                                                                                  # take the refinement points
        if len(points) == 0:                                                                                                # if no basepoints, than no sort
            self.logger.warning('There are no {0}points to sort'.format(modeltype))
            return
        westSide = []                                                                                                       # split west and east side of pier
        eastSide = []                                                                                                       # and reset them
        a = sorted(points, key=operator.itemgetter(0))                                                                      # first sort for az
        for i in range(0, len(a)):                                                                                          # split flip sides
            if a[i][0] >= 180:                                                                                              # choose the right side
                westSide.append((a[i][0], a[i][1]))                                                                         # add the point tto list
            else:                                                                                                           #
                eastSide.append((a[i][0], a[i][1]))                                                                         #
        westSide = sorted(westSide, key=operator.itemgetter(1))                                                             # sort west flipside
        eastSide = sorted(eastSide, key=operator.itemgetter(1))                                                             # sort east flipside
        if modeltype == 'base':
            self.BasePoints = eastSide + westSide                                                                           # put them together
        else:
            self.RefinementPoints = eastSide + westSide                                                                     # put them together

    def loadHorizonPoints(self, horizonPointsFileName, file_check, line_check, line_value):                                 # load a ModelMaker modeling file, return base & refine points as lists of (az,alt) tuples
        self.horizonPoints = []                                                                                             # clear horizon variable
        hp = []                                                                                                             # clear cache
        msg = None
        minAlt = 0
        if file_check:
            if horizonPointsFileName == '':
                msg = 'No horizon points filename given !'
                return msg
            if not os.path.isfile(os.getcwd() + '/config/' + horizonPointsFileName):
                msg = 'Horizon points file does not exist !'
                self.logger.warning('horizon points file does not exist')
            else:
                try:                                                                                                        # try opening the file
                    with open(os.getcwd() + '/config/' + horizonPointsFileName) as f:                                       # run through file
                        for line in f:                                                                                      # run through lines
                            if ':' in line:
                                m = line.rstrip('\n').split(':')                                                            # model maker format
                            else:
                                m = line.rstrip('\n').split(' ')                                                            # card du ciel format
                            point = (int(m[0]), int(m[1]))                                                                  # get point data
                            hp.append(point)                                                                                # add the point
                    f.close()                                                                                               # close file again
                except Exception as e:                                                                                      # handle exception
                    msg = 'Error loading horizon points: {0}'.format(e)
                    self.logger.error('Error loading horizon points: {0}'.format(e))
                    return msg                                                                                              # stop routine
            hp = sorted(hp, key=operator.itemgetter(0))                                                                     # list should be sorted, but I do it for security anyway
        if line_check:
            minAlt = int(line_value)
            if len(hp) == 0:                                                                                                # there is no file loaded
                hp = ((0, minAlt), (359, minAlt))
        az_last = 0                                                                                                         # starting azimuth
        alt_last = 0                                                                                                        # starting altitude
        for i in range(0, len(hp)):                                                                                         # run through all points an link them via line
            az_act = hp[i][0]                                                                                               # get az from point of file
            alt_act = hp[i][1]                                                                                              # get alt from point of file
            if az_act > az_last:                                                                                            # if act az is greater than last on, we have to draw a line
                incline = (alt_act - alt_last) / (az_act - az_last)                                                         # calculation the line incline
                for j in range(az_last, az_act):                                                                            # run through the space, where no points are given in the file
                    if line_check:
                        point = (j, max(int(alt_last + incline * (j - az_last)), minAlt))                                    # calculation value of next point
                    else:
                        point = (j, int(alt_last + incline * (j - az_last)))
                    self.horizonPoints.append(point)                                                                        # add the interpolated point to list
            else:                                                                                                           # otherwise no interpolation
                self.horizonPoints.append(hp[i])                                                                            # add the point to list, no interpolation needed
            az_last = az_act                                                                                                # set az value to next point
            alt_last = alt_act                                                                                              # same to alt value
        return msg

    def isAboveHorizonLine(self, point):                                                                                    # check, if point is above horizon list (by horizon file)
        if len(self.horizonPoints) > 0:                                                                                     # check if there are horizon points
            if point[1] > self.horizonPoints[int(point[0])][1]:                                                             # simple comparison. important: each Int(az) has value set
                return True                                                                                                 # point is above -> True
            else:                                                                                                           #
                return False                                                                                                # points are below the horizon line -> False
        else:                                                                                                               #
            return True                                                                                                     # if there is no horizon line, all are in

    def deleteBelowHorizonLine(self):                                                                                       # remove points from modeling, if below horizon
        i = 0                                                                                                               # list index
        while i < len(self.RefinementPoints):                                                                               # loop for generating list index of point which is below horizon
            if self.isAboveHorizonLine(self.RefinementPoints[i]):                                                           #
                i += 1                                                                                                      # if ok , keep him
            else:                                                                                                           #
                del self.RefinementPoints[i]                                                                                # otherwise delete point from list

    def deletePoints(self):
        self.BasePoints = []
        self.RefinementPoints = []

    def loadBasePoints(self, filename):
        self.BasePoints, msg = self.loadModelPoints(filename, 'base')

    def loadRefinementPoints(self, filename):
        self.RefinementPoints, msg = self.loadModelPoints(filename, 'refinement')

    def generateDSOPoints(self, hours, numPoints, hoursPrev, ra, dec):                                                      # modeling points along dso path
        self.RefinementPoints = []                                                                                          # clear point list
        for i in range(0, numPoints):                                                                                       # round modeling point from actual az alt position 24 hours
            ra = ra - float(i) * hours / numPoints - hoursPrev
            az, alt = self.transform.transformERFA(ra, dec, 1)                                                             # transform to az alt
            if alt > 0:                                                                                                     # we only take point alt > 0
                self.RefinementPoints.append((az, alt))                                                                     # add point to list

    def generateDensePoints(self):                                                                                          # generate pointcloud in greater circles of sky
        west = []                                                                                                           # no sorting, point will be for west and east prepared
        east = []                                                                                                           #
        for dec in range(-10, 90, 10):                                                                                      # range, actually referenced from european situation
            if dec < 30:                                                                                                    # has to be generalized
                step = -15                                                                                                  # lower dec, more point
            elif dec < 70:
                step = -10
            else:
                step = -30                                                                                                  # higher dec. less point (anyway denser)
            for ha in range(120, -120, step):                                                                               # for complete 24 hourangle
                az, alt = self.transform.transformERFA(ha / 10, dec, 1)                                                    # do the transformation to alt az
                if alt > 0:                                                                                                 # only point with alt > 0 are taken
                    if az > 180:                                                                                            # put to the right list
                        east.append((az, alt))                                                                              # add to east
                    else:
                        west.append((az, alt))                                                                              # add to west
        self.RefinementPoints = west + east

    def generateNormalPoints(self):
        west = []                                                                                                           # no sorting, point will be for west and east prepared
        east = []                                                                                                           #
        for dec in range(-15, 90, 15):                                                                                      # range, actually referenced from european situation
            if dec < 60:                                                                                                    # has to be generalized
                step = -10                                                                                                  # lower dec, more point
            else:
                step = -20                                                                                                  # higher dec. less point (anyway denser)
            for ha in range(120, -120, step):                                                                               # for complete 24 hourangle
                az, alt = self.transform.transformERFA(ha / 10, dec, 1)                                                    # do the transformation to alt az
                if alt > 0:                                                                                                 # only point with alt > 0 are taken
                    if az > 180:                                                                                            # put to the right list
                        east.append((az, alt))                                                                              # add to east
                    else:
                        west.append((az, alt))                                                                              # add to west
        self.RefinementPoints = west + east

    def generateGridPoints(self, row, col, altMin, altMax):                                                                                           # modeling points along dso path
        self.RefinementPoints = []                                                                                          # clear point list
        for az in range(5, 360, int(360 / col)):                                                                            # make point for all azimuth
            for alt in range(altMin, altMax + 1, int((altMax - altMin) / (row - 1))):                                       # make point for all altitudes
                self.RefinementPoints.append((az, alt))                                                                     # add point to list

    def generateBasePoints(self, az, alt):                                                                                           # do base point equally distributed
        self.BasePoints = []
        for i in range(0, 3):                                                                                               # we need 3 basepoints
            azp = i * 120 + az                                                                                              # equal distance of 120 degree in az
            if azp > 360:                                                                                                   # value range 0-360
                azp -= 360                                                                                                  # shift it if necessary
            point = (azp, alt)                                                                                              # generate the point value az,alt
            self.BasePoints.append(point)                                                                                   # put it to list



import time
from unittest import TestCase

from astropy import _erfa as erfa_astro

from astrometry import erfa


class TestErfa(TestCase):

    def setUp(self):
        # necessary structs in classes
        self.ERFA = erfa.ERFA()

        # show performance
        self.show_perf = 1
        # parameters for test

        self.ts = time.gmtime(0)

    def test_eraJd2cal(self):
        dj1 = 2458027.31896727 + 1.5
        dj2 = 0.0 - 0.81896727019

        start_time = time.clock()
        ok, iy, im, id, fd = self.ERFA.eraJd2cal(dj1, dj2)
        if self.show_perf:
            print(" ERFA Jd2cal     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        value = erfa_astro.jd2cal(dj1, dj2)
        if self.show_perf:
            print(" ERFA Jd2cal     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(ok, 0)
        self.assertEqual(value[0], iy)
        self.assertEqual(value[1], im)
        self.assertEqual(value[2], id)
        self.assertAlmostEqual(value[3], fd)

        dj2 = 0.0
        for j in range(2450000, 2460000):
            dj1 = j / 10

            ok, iy, im, id, fd = self.ERFA.eraJd2cal(dj1, dj2)
            value = erfa_astro.jd2cal(dj1, dj2)

            self.assertEqual(ok, 0)
            self.assertEqual(value[0], iy)
            self.assertEqual(value[1], im)
            self.assertEqual(value[2], id)
            self.assertAlmostEqual(value[3], fd)

        dj1 = 2400000.5
        dj2 = 50123.2
        ok, iy, im, id, fd = self.ERFA.eraJd2cal(dj1, dj2)
        value = erfa_astro.jd2cal(dj1, dj2)
        self.assertEqual(ok, 0)
        self.assertEqual(value[0], iy)
        self.assertEqual(value[1], im)
        self.assertEqual(value[2], id)
        self.assertAlmostEqual(value[3], fd)

        dj1 = 2451545.0
        dj2 = -1421.3
        ok, iy, im, id, fd = self.ERFA.eraJd2cal(dj1, dj2)
        value = erfa_astro.jd2cal(dj1, dj2)
        self.assertEqual(ok, 0)
        self.assertEqual(value[0], iy)
        self.assertEqual(value[1], im)
        self.assertEqual(value[2], id)
        self.assertAlmostEqual(value[3], fd)

        dj1 = 2450123.5
        dj2 = 0.2
        ok, iy, im, id, fd = self.ERFA.eraJd2cal(dj1, dj2)
        value = erfa_astro.jd2cal(dj1, dj2)
        self.assertEqual(ok, 0)
        self.assertEqual(value[0], iy)
        self.assertEqual(value[1], im)
        self.assertEqual(value[2], id)
        self.assertAlmostEqual(value[3], fd)

        dj1 = 2451967
        dj2 = 0
        ok, iy, im, id, fd = self.ERFA.eraJd2cal(dj1, dj2)
        value = erfa_astro.jd2cal(dj1, dj2)
        self.assertEqual(ok, 0)
        self.assertEqual(value[0], iy)
        self.assertEqual(value[1], im)
        self.assertEqual(value[2], id)
        self.assertAlmostEqual(value[3], fd)

    def test_eraCal2jd(self):
        start_time = time.clock()
        ok, djm0, djm = self.ERFA.eraCal2jd(self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday)
        if self.show_perf:
            print(" ERFA Cal2jd     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        value = erfa_astro.cal2jd(self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday)
        if self.show_perf:
            print(" ERFA Cal2jd     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(ok, 0)
        self.assertEqual(value[0], djm0)
        self.assertEqual(value[1], djm)

    def test_eraDtf2d(self):
        start_time = time.clock()
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        if self.show_perf:
            print(" ERFA Dtf2d      Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        value = erfa_astro.dtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        if self.show_perf:
            print(" ERFA Dtf2d      Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(0, ok)
        self.assertEqual(date1 + date2, value[0])

    def test_eraEpv00(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)

        start_time = time.clock()
        ok, pvh, pvb = self.ERFA.eraEpv00(date1, date2)
        if self.show_perf:
            print(" ERFA Epv00      Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        pvh_ref, pvb_ref = erfa_astro.epv00(date1, date2)
        if self.show_perf:
            print(" ERFA Epv00      Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(ok, 0)
        self.assertEqual(pvh[0][0], pvh_ref[0][0])
        self.assertEqual(pvh[0][1], pvh_ref[0][1])
        self.assertEqual(pvh[0][2], pvh_ref[0][2])
        self.assertEqual(pvh[1][0], pvh_ref[1][0])
        self.assertEqual(pvh[1][1], pvh_ref[1][1])
        self.assertEqual(pvh[1][2], pvh_ref[1][2])
        self.assertEqual(pvb[0][0], pvb_ref[0][0])
        self.assertEqual(pvb[0][1], pvb_ref[0][1])
        self.assertEqual(pvb[0][2], pvb_ref[0][2])
        self.assertEqual(pvb[1][0], pvb_ref[1][0])
        self.assertEqual(pvb[1][1], pvb_ref[1][1])
        self.assertEqual(pvb[1][2], pvb_ref[1][2])

    def test_eraObl06(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)

        start_time = time.clock()
        epsa = self.ERFA.eraObl06(date1, date2)
        if self.show_perf:
            print(" ERFA Obl06      Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        epsa_ref = erfa_astro.obl06(date1, date2)
        if self.show_perf:
            print(" ERFA Obl06      Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(epsa, epsa_ref)

    def test_eraPfw06(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)

        start_time = time.clock()
        gamb, phib, psib, epsa = self.ERFA.eraPfw06(date1, date2)
        if self.show_perf:
            print(" ERFA Pfw06      Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        gamb_ref, phib_ref, psib_ref, epsa_ref = erfa_astro.pfw06(date1, date2)
        if self.show_perf:
            print(" ERFA Pfw06      Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(gamb, gamb_ref)
        self.assertEqual(phib, phib_ref)
        self.assertEqual(psib, psib_ref)
        self.assertEqual(epsa, epsa_ref)

    def test_eraFal03(self):
        start_time = time.clock()
        self.ERFA.eraFal03(1)
        if self.show_perf:
            print(" ERFA Fal03      Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        erfa_astro.fal03(1)
        if self.show_perf:
            print(" ERFA Fal03      Astropy {0:15.15f} seconds".format((time.clock() - start_time)))
        for i in range(-10, 10):
            el = self.ERFA.eraFal03(i)
            el_ref = erfa_astro.fal03(i)
            self.assertEqual(el, el_ref)

    def test_eraFaf03(self):
        start_time = time.clock()
        self.ERFA.eraFaf03(1)
        if self.show_perf:
            print(" ERFA Faf03      Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        erfa_astro.faf03(1)
        if self.show_perf:
            print(" ERFA Faf03      Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        for i in range(-10, 10):
            el = self.ERFA.eraFaf03(i)
            el_ref = erfa_astro.faf03(i)
            self.assertEqual(el, el_ref)

    def test_eraFaom03(self):
        start_time = time.clock()
        self.ERFA.eraFaom03(1)
        if self.show_perf:
            print(" ERFA Faom03     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        erfa_astro.faom03(1)
        if self.show_perf:
            print(" ERFA Faom03     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        for i in range(-10, 10):
            el_ref = erfa_astro.faom03(i)
            el = self.ERFA.eraFaom03(i)
            self.assertEqual(el, el_ref)

    def test_eraNut00a(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)

        start_time = time.clock()
        dp, de = self.ERFA.eraNut00a(date1, date2)
        if self.show_perf:
            print(" ERFA Nut00a     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        dp_ref, de_ref = erfa_astro.nut00a(date1, date2)
        if self.show_perf:
            print(" ERFA Nut00a     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(dp, dp_ref)
        self.assertEqual(de, de_ref)

    def test_eraNut06a(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)

        start_time = time.clock()
        dp, de = self.ERFA.eraNut06a(date1, date2)
        if self.show_perf:
            print(" ERFA Nut06a     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        dp_ref, de_ref = erfa_astro.nut06a(date1, date2)
        if self.show_perf:
            print(" ERFA Nut06a     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(dp, dp_ref)
        self.assertEqual(de, de_ref)

    def test_eraPnm06a(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)

        start_time = time.clock()
        rnpb = self.ERFA.eraPnm06a(date1, date2)
        if self.show_perf:
            print(" ERFA Pnm06a     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        rnpb_ref = erfa_astro.pnm06a(date1, date2)
        if self.show_perf:
            print(" ERFA Pnm06a     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(rnpb[0][0], rnpb_ref[0][0])
        self.assertEqual(rnpb[0][1], rnpb_ref[0][1])
        self.assertEqual(rnpb[0][2], rnpb_ref[0][2])
        self.assertEqual(rnpb[1][0], rnpb_ref[1][0])
        self.assertEqual(rnpb[1][1], rnpb_ref[1][1])
        self.assertEqual(rnpb[1][2], rnpb_ref[1][2])
        self.assertEqual(rnpb[2][0], rnpb_ref[2][0])
        self.assertEqual(rnpb[2][1], rnpb_ref[2][1])
        self.assertEqual(rnpb[2][2], rnpb_ref[2][2])

    def test_eraBpn2xy(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        rnpb = self.ERFA.eraPnm06a(date1, date2)

        start_time = time.clock()
        x, y = self.ERFA.eraBpn2xy(rnpb)
        if self.show_perf:
            print(" ERFA Bpn2xy     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        x_ref, y_ref = erfa_astro.bpn2xy(rnpb)
        if self.show_perf:
            print(" ERFA Bpn2xy     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(x, x_ref)
        self.assertEqual(y, y_ref)

    def test_eraS06(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        rnpb = self.ERFA.eraPnm06a(date1, date2)
        x, y = self.ERFA.eraBpn2xy(rnpb)

        start_time = time.clock()
        val = self.ERFA.eraS06(date1, date2, x, y)
        if self.show_perf:
            print(" ERFA S06        Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        val_ref = erfa_astro.s06(date1, date2, x, y)
        if self.show_perf:
            print(" ERFA S06        Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(val, val_ref)

    def test_eraApci(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        ok, ehpv, ebpv = self.ERFA.eraEpv00(date1, date2)
        rnpb = self.ERFA.eraPnm06a(date1, date2)
        x, y = self.ERFA.eraBpn2xy(rnpb)
        s = self.ERFA.eraS06(date1, date2, x, y)

        start_time = time.clock()
        self.ERFA.eraApci(date1, date2, ebpv,  ehpv[0], x, y, s)
        if self.show_perf:
            print(" ERFA Apci       Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        astrom_ref = erfa_astro.apci(date1, date2, ebpv,  ehpv[0], x, y, s)
        if self.show_perf:
            print(" ERFA Apci       Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(astrom_ref[()][6][0][0], self.ERFA.astrom.bpn[0][0])
        self.assertEqual(astrom_ref[()][6][0][1], self.ERFA.astrom.bpn[0][1])
        self.assertEqual(astrom_ref[()][6][0][2], self.ERFA.astrom.bpn[0][2])
        self.assertEqual(astrom_ref[()][6][1][0], self.ERFA.astrom.bpn[1][0])
        self.assertEqual(astrom_ref[()][6][1][1], self.ERFA.astrom.bpn[1][1])
        self.assertEqual(astrom_ref[()][6][1][2], self.ERFA.astrom.bpn[1][2])
        self.assertEqual(astrom_ref[()][6][2][0], self.ERFA.astrom.bpn[2][0])
        self.assertEqual(astrom_ref[()][6][2][1], self.ERFA.astrom.bpn[2][1])
        self.assertEqual(astrom_ref[()][6][2][2], self.ERFA.astrom.bpn[2][2])

    def test_eraEors(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        ok, ehpv, ebpv = self.ERFA.eraEpv00(date1, date2)
        rnpb = self.ERFA.eraPnm06a(date1, date2)
        x, y = self.ERFA.eraBpn2xy(rnpb)
        s = self.ERFA.eraS06(date1, date2, x, y)
        self.ERFA.eraApci(date1, date2, ebpv,  ehpv[0], x, y, s)

        start_time = time.clock()
        eo = self.ERFA.eraEors(rnpb, s)
        if self.show_perf:
            print(" ERFA Eors       Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        eo_ref = erfa_astro.eors(rnpb, s)
        if self.show_perf:
            print(" ERFA Eors       Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(eo, eo_ref)

    def test_eraApci13(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)

        start_time = time.clock()
        eo = self.ERFA.eraApci13(date1, date2)
        if self.show_perf:
            print(" ERFA Apci13     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        astrom_ref, eo_ref = erfa_astro.apci13(date1, date2)
        if self.show_perf:
            print(" ERFA Apci13     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(astrom_ref[()][0], self.ERFA.astrom.pmt)
        self.assertEqual(astrom_ref[()][1][0], self.ERFA.astrom.eb[0])
        self.assertEqual(astrom_ref[()][1][1], self.ERFA.astrom.eb[1])
        self.assertEqual(astrom_ref[()][1][2], self.ERFA.astrom.eb[2])
        self.assertEqual(astrom_ref[()][2][0], self.ERFA.astrom.eh[0])
        self.assertEqual(astrom_ref[()][2][1], self.ERFA.astrom.eh[1])
        self.assertEqual(astrom_ref[()][2][2], self.ERFA.astrom.eh[2])
        self.assertEqual(astrom_ref[()][3], self.ERFA.astrom.em)
        self.assertEqual(astrom_ref[()][4][0], self.ERFA.astrom.v[0])
        self.assertEqual(astrom_ref[()][4][1], self.ERFA.astrom.v[1])
        self.assertEqual(astrom_ref[()][4][2], self.ERFA.astrom.v[2])
        self.assertEqual(astrom_ref[()][5], self.ERFA.astrom.bml)
        self.assertEqual(astrom_ref[()][6][0][0], self.ERFA.astrom.bpn[0][0])
        self.assertEqual(astrom_ref[()][6][0][1], self.ERFA.astrom.bpn[0][1])
        self.assertEqual(astrom_ref[()][6][0][2], self.ERFA.astrom.bpn[0][2])
        self.assertEqual(astrom_ref[()][6][1][0], self.ERFA.astrom.bpn[1][0])
        self.assertEqual(astrom_ref[()][6][1][1], self.ERFA.astrom.bpn[1][1])
        self.assertEqual(astrom_ref[()][6][1][2], self.ERFA.astrom.bpn[1][2])
        self.assertEqual(astrom_ref[()][6][2][0], self.ERFA.astrom.bpn[2][0])
        self.assertEqual(astrom_ref[()][6][2][1], self.ERFA.astrom.bpn[2][1])
        self.assertEqual(astrom_ref[()][6][2][2], self.ERFA.astrom.bpn[2][2])
        self.assertAlmostEqual(astrom_ref[()][7], self.ERFA.astrom.along)
        self.assertAlmostEqual(astrom_ref[()][8], self.ERFA.astrom.phi)
        self.assertAlmostEqual(astrom_ref[()][9], self.ERFA.astrom.xpl)
        self.assertAlmostEqual(astrom_ref[()][10], self.ERFA.astrom.ypl)
        self.assertAlmostEqual(astrom_ref[()][11], self.ERFA.astrom.sphi)
        self.assertAlmostEqual(astrom_ref[()][12], self.ERFA.astrom.cphi)
        self.assertAlmostEqual(astrom_ref[()][13], self.ERFA.astrom.diurab)
        self.assertAlmostEqual(astrom_ref[()][14], self.ERFA.astrom.eral)
        self.assertAlmostEqual(astrom_ref[()][15], self.ERFA.astrom.refa)
        self.assertAlmostEqual(astrom_ref[()][16], self.ERFA.astrom.refb)
        self.assertAlmostEqual(eo_ref, eo)

    def test_eraPmpx(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        self.ERFA.eraApci13(date1, date2)
        rc = 3.14
        dc = 0.5
        pr = 0
        pd = 0
        px = 0
        rv = 0

        start_time = time.clock()
        pco = self.ERFA.eraPmpx(rc, dc, pr, pd, px, rv, self.ERFA.astrom.pmt, self.ERFA.astrom.eb)
        if self.show_perf:
            print(" ERFA Pmpx       Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        pco_ref = erfa_astro.pmpx(rc, dc, pr, pd, px, rv, self.ERFA.astrom.pmt, self.ERFA.astrom.eb)
        if self.show_perf:
            print(" ERFA Pmpx       Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(pco[0], pco_ref[0])
        self.assertEqual(pco[1], pco_ref[1])
        self.assertEqual(pco[2], pco_ref[2])

    def test_eraLdsun(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        self.ERFA.eraApci13(date1, date2)
        rc = 3.14
        dc = 0.5
        pr = 0
        pd = 0
        px = 0
        rv = 0
        pco = self.ERFA.eraPmpx(rc, dc, pr, pd, px, rv, self.ERFA.astrom.pmt, self.ERFA.astrom.eb)

        start_time = time.clock()
        pnat = self.ERFA.eraLdsun(pco, self.ERFA.astrom.eh, self.ERFA.astrom.em)
        if self.show_perf:
            print(" ERFA Ldsun      Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        pnat_ref = erfa_astro.ldsun(pco, self.ERFA.astrom.eh, self.ERFA.astrom.em)
        if self.show_perf:
            print(" ERFA Ldsun      Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(pnat[0], pnat_ref[0])
        self.assertEqual(pnat[1], pnat_ref[1])
        self.assertEqual(pnat[2], pnat_ref[2])

    def test_eraAb(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        self.ERFA.eraApci13(date1, date2)
        rc = 3.14
        dc = 0.5
        pr = 0
        pd = 0
        px = 0
        rv = 0
        pco = self.ERFA.eraPmpx(rc, dc, pr, pd, px, rv, self.ERFA.astrom.pmt, self.ERFA.astrom.eb)
        pnat = self.ERFA.eraLdsun(pco, self.ERFA.astrom.eh, self.ERFA.astrom.em)

        start_time = time.clock()
        ppr = self.ERFA.eraAb(pnat, self.ERFA.astrom.v, self.ERFA.astrom.em, self.ERFA.astrom.bml)
        if self.show_perf:
            print(" ERFA Ab         Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        ppr_ref = erfa_astro.ab(pnat, self.ERFA.astrom.v, self.ERFA.astrom.em, self.ERFA.astrom.bml)
        if self.show_perf:
            print(" ERFA Ab         Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(ppr[0], ppr_ref[0])
        self.assertEqual(ppr[1], ppr_ref[1])
        self.assertEqual(ppr[2], ppr_ref[2])

    def test_eraAtci13(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        rc = 3.14
        dc = 0.5
        pr = 0
        pd = 0
        px = 0
        rv = 0

        start_time = time.clock()
        ri, di, eo = self.ERFA.eraAtci13(rc, dc, pr, pd, px, rv, date1, date2)
        if self.show_perf:
            print(" ERFA Atci13     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        ri_ref, di_ref, eo_ref = erfa_astro.atci13(rc, dc, pr, pd, px, rv, date1, date2)
        if self.show_perf:
            print(" ERFA Atci13     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(ri, ri_ref)
        self.assertEqual(di, di_ref)
        self.assertEqual(eo, eo_ref)

    def test_eraAtic13(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        ri = 03.84027 * self.ERFA.ERFA_D2PI / 24
        di = 61.3768 * self.ERFA.ERFA_D2PI / 360

        start_time = time.clock()
        rc, dc, eo = self.ERFA.eraAtic13(ri, di, date1, date2)
        if self.show_perf:
            print(" ERFA Atic13     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        rc_ref, dc_ref, eo_ref = erfa_astro.atic13(ri, di, date1, date2)
        if self.show_perf:
            print(" ERFA Atic13     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(rc, rc_ref)
        self.assertEqual(dc, dc_ref)
        self.assertEqual(eo, eo_ref)

    def test_eraAtioq(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        r = 1.5
        d = 2.0
        eo = self.ERFA.eraApci13(date1, date2)
        astrom_ref, eo_ref = erfa_astro.apci13(date1, date2)

        start_time = time.clock()
        aob, zob, hob, dob, rob = self.ERFA.eraAtioq(r, d)
        if self.show_perf:
            print(" ERFA Atioq      Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        aob_ref, zob_ref, hob_ref, dob_ref, rob_ref = erfa_astro.atioq(r, d, astrom_ref)
        if self.show_perf:
            print(" ERFA Atioq      Astropy {0:15.15f} seconds".format((time.clock() - start_time)))
        self.assertEqual(aob, aob_ref)
        self.assertEqual(zob, zob_ref)
        self.assertEqual(hob, hob_ref)
        self.assertEqual(dob, dob_ref)
        self.assertEqual(rob, rob_ref)

    def test_eraUtctai(self):
        ok, utc1, utc2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)

        start_time = time.clock()
        ok, tai1, tai2 = self.ERFA.eraUtctai(utc1, utc2)
        if self.show_perf:
            print(" ERFA Utctai     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        tai1_ref, tai2_ref = erfa_astro.utctai(utc1, utc2)
        if self.show_perf:
            print(" ERFA Utctai     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(ok, 0)
        self.assertEqual(tai1, tai1_ref)
        self.assertEqual(tai2, tai2_ref)

    def test_eraUtcutl(self):
        ok, utc1, utc2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        dt = 30

        start_time = time.clock()
        ok, ut11, ut12 = self.ERFA.eraUtcut1(utc1, utc2, dt)
        if self.show_perf:
            print(" ERFA Utcutl     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        ut11_ref, ut12_ref = erfa_astro.utcut1(utc1, utc2, dt)
        if self.show_perf:
            print(" ERFA Utcutl     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(ok, 0)
        self.assertEqual(ut11, ut11_ref)
        self.assertEqual(ut12, ut12_ref)

    def test_eraRefco(self):
        phpa = 1000
        tc = 20
        rh = 0.8
        wl = 0.57

        start_time = time.clock()
        refa, refb = self.ERFA.eraRefco(phpa, tc, rh, wl)
        if self.show_perf:
            print(" ERFA Refco      Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        refa_ref, refb_ref = erfa_astro.refco(phpa, tc, rh, wl)
        if self.show_perf:
            print(" ERFA Refco      Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(refa, refa_ref)
        self.assertEqual(refb, refb_ref)

    def test_eraApco13(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        elong = 11.0 * self.ERFA.ERFA_DD2R
        phi = 49.0 * self.ERFA.ERFA_DD2R
        hm = 583.0
        xp = 0
        yp = 0
        j, dut1 = self.ERFA.eraDat(self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, 0)
        phpa = 1000
        tc = 20
        rh = 0.8
        wl = 0.57

        start_time = time.clock()
        j, eo = self.ERFA.eraApco13(date1, date2, dut1, elong, phi, hm, xp, yp, phpa, tc, rh, wl)
        if self.show_perf:
            print(" ERFA Apco13     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        astrom_ref, eo_ref = erfa_astro.apco13(date1, date2, dut1, elong, phi, hm, xp, yp, phpa, tc, rh, wl)
        if self.show_perf:
            print(" ERFA Apco13     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(astrom_ref[()][0], self.ERFA.astrom.pmt)
        self.assertEqual(astrom_ref[()][1][0], self.ERFA.astrom.eb[0])
        self.assertEqual(astrom_ref[()][1][1], self.ERFA.astrom.eb[1])
        self.assertEqual(astrom_ref[()][1][2], self.ERFA.astrom.eb[2])
        self.assertEqual(astrom_ref[()][2][0], self.ERFA.astrom.eh[0])
        self.assertEqual(astrom_ref[()][2][1], self.ERFA.astrom.eh[1])
        self.assertEqual(astrom_ref[()][2][2], self.ERFA.astrom.eh[2])
        self.assertEqual(astrom_ref[()][3], self.ERFA.astrom.em)
        self.assertEqual(astrom_ref[()][4][0], self.ERFA.astrom.v[0])
        self.assertEqual(astrom_ref[()][4][1], self.ERFA.astrom.v[1])
        self.assertEqual(astrom_ref[()][4][2], self.ERFA.astrom.v[2])
        self.assertEqual(astrom_ref[()][5], self.ERFA.astrom.bml)
        self.assertEqual(astrom_ref[()][6][0][0], self.ERFA.astrom.bpn[0][0])
        self.assertEqual(astrom_ref[()][6][0][1], self.ERFA.astrom.bpn[0][1])
        self.assertEqual(astrom_ref[()][6][0][2], self.ERFA.astrom.bpn[0][2])
        self.assertEqual(astrom_ref[()][6][1][0], self.ERFA.astrom.bpn[1][0])
        self.assertEqual(astrom_ref[()][6][1][1], self.ERFA.astrom.bpn[1][1])
        self.assertEqual(astrom_ref[()][6][1][2], self.ERFA.astrom.bpn[1][2])
        self.assertEqual(astrom_ref[()][6][2][0], self.ERFA.astrom.bpn[2][0])
        self.assertEqual(astrom_ref[()][6][2][1], self.ERFA.astrom.bpn[2][1])
        self.assertEqual(astrom_ref[()][6][2][2], self.ERFA.astrom.bpn[2][2])
        self.assertAlmostEqual(astrom_ref[()][7], self.ERFA.astrom.along)
        self.assertAlmostEqual(astrom_ref[()][8], self.ERFA.astrom.phi)
        self.assertAlmostEqual(astrom_ref[()][9], self.ERFA.astrom.xpl)
        self.assertAlmostEqual(astrom_ref[()][10], self.ERFA.astrom.ypl)
        self.assertAlmostEqual(astrom_ref[()][11], self.ERFA.astrom.sphi)
        self.assertAlmostEqual(astrom_ref[()][12], self.ERFA.astrom.cphi)
        self.assertAlmostEqual(astrom_ref[()][13], self.ERFA.astrom.diurab)
        self.assertAlmostEqual(astrom_ref[()][14], self.ERFA.astrom.eral)
        self.assertAlmostEqual(astrom_ref[()][15], self.ERFA.astrom.refa)
        self.assertAlmostEqual(astrom_ref[()][16], self.ERFA.astrom.refb)
        self.assertAlmostEqual(eo_ref, eo)

    def test_eraAtoc13(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        elong = 11.0 * self.ERFA.ERFA_DD2R
        phi = 49.0 * self.ERFA.ERFA_DD2R
        hm = 583.0
        xp = 0
        yp = 0
        j, dut1 = self.ERFA.eraDat(self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, 0)
        phpa = 1000
        tc = 20
        rh = 0.8
        wl = 0.57
        ob1 = 3.14
        ob2 = 0.5
        type_ERA = 'A'

        start_time = time.clock()
        j, rc, dc = self.ERFA.eraAtoc13(type_ERA, ob1, ob2, date1, date2, dut1, elong, phi, hm, xp, yp, phpa, tc, rh, wl)
        if self.show_perf:
            print(" ERFA Atoc13     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        rc_ref, dc_ref = erfa_astro.atoc13(type_ERA, ob1, ob2, date1, date2, dut1, elong, phi, hm, xp, yp, phpa, tc, rh, wl)
        if self.show_perf:
            print(" ERFA Atoc13     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))

        self.assertEqual(rc, rc_ref)
        self.assertEqual(dc, dc_ref)
        self.assertEqual(ok, 0)

    def test_eraAtco13(self):
        ok, date1, date2 = self.ERFA.eraDtf2d('UTC', self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, self.ts.tm_hour, self.ts.tm_min, self.ts.tm_sec)
        elong = 11.0 * self.ERFA.ERFA_DD2R
        phi = 49.0 * self.ERFA.ERFA_DD2R
        hm = 583.0
        xp = 0
        yp = 0
        j, dut1 = self.ERFA.eraDat(self.ts.tm_year, self.ts.tm_mon, self.ts.tm_mday, 0)
        phpa = 1000
        tc = 20
        rh = 0.8
        wl = 0.57
        rc = 20 * self.ERFA.ERFA_DD2R * 24 / 360
        dc = 0 * self.ERFA.ERFA_DD2R
        pr = 0
        pd = 0
        px = 0
        rv = 0

        start_time = time.clock()
        ok, aob, zob, hob, dob, rob, eo = self.ERFA.eraAtco13(rc, dc, pr, pd, px, rv, date1, date2, dut1, elong, phi, hm, xp, yp, phpa, tc, rh, wl)
        if self.show_perf:
            print(" ERFA Apco13     Python  {0:15.15f} seconds".format((time.clock() - start_time)))

        start_time = time.clock()
        aob_ref, zob_ref, hob_ref, dob_ref, rob_ref, eo_ref = erfa_astro.atco13(rc, dc, pr, pd, px, rv, date1, date2, dut1, elong, phi, hm, xp, yp, phpa, tc, rh, wl)
        if self.show_perf:
            print(" ERFA Apco13     Astropy {0:15.15f} seconds".format((time.clock() - start_time)))
        self.assertEqual(ok, 0)
        self.assertEqual(aob, aob_ref)
        self.assertEqual(zob, zob_ref)
        self.assertEqual(hob, hob_ref)
        self.assertEqual(dob, dob_ref)
        self.assertEqual(rob, rob_ref)
        self.assertEqual(eo, eo_ref)




############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################

# import basic stuff
import logging
import threading

import pythoncom
from win32com.client.dynamic import Dispatch

# astrometry
from astrometry import transform


class MountAscom:
    logger = logging.getLogger(__name__)                                                                                    # enable logging

    def __init__(self, app):
        self.app = app
        self.ascom = None                                                                                                   # ascom mount driver entry point
        self.connected = False                                                                                              # init of connection status
        self.driver_real = True
        self.driverName = 'ASCOM.FrejvallGM.Telescope'                                                                      # default driver name is Per's driver
        self.chooser = None                                                                                                 # object space
        self.value_azimuth = 0
        self.value_altitude = 0
        self.sendCommandLock = threading.Lock()

    def connect(self):                                                                                                      # runnable of the thread
        try:
            self.ascom = Dispatch(self.driverName)                                                                          # select win32 driver
            if self.driverName == 'ASCOM.FrejvallGM.Telescope' or self.driverName == 'ASCOM.tenmicron_mount.Telescope':     # identify real telescope against simulator
                self.driver_real = True                                                                                     # set it
            else:
                self.driver_real = False                                                                                    # set it
            self.ascom.connected = True                                                                                     # connect to mount
            self.connected = True                                                                                           # setting connection status from driver
        except Exception as e:                                                                                              # error handling
            self.logger.error('Driver COM Error in dispatchMount: {0}'.format(e))
            self.connected = False                                                                                          # connection broken
        finally:                                                                                                            # we don't stop, but try it again
            pass

    def disconnect(self):
        try:
            self.connected = False
            self.ascom.connected = False                                                                                    # connect to mount
            self.ascom.Quit()
            self.ascom = None
        except Exception as e:                                                                                              # error handling
            self.logger.error('Driver COM Error in dispatchMount: {0}'.format(e))
            self.connected = False                                                                                          # connection broken
        finally:                                                                                                            # we don't stop, but try it again
            pass

    def sendCommand(self, command):                                                                                         # core routine for sending commands to mount
        reply = ''                                                                                                          # reply is empty
        value = '0'
        self.sendCommandLock.acquire()
        if self.driver_real and self.connected:
            try:                                                                                                            # all with error handling
                if command in self.app.mount.BLIND_COMMANDS:                                                                # these are the commands, which do not expect a return value
                    self.ascom.CommandBlind(command)                                                                        # than do blind command
                else:                                                                                                       #
                    reply = self.ascom.CommandString(command)                                                               # with return value do regular command
            except pythoncom.com_error as e:                                                                                # error handling
                self.app.messageQueue.put('Driver COM Error in sendCommand')                                                # gui
                self.logger.error('error: {0} command:{1}  reply:{2} '.format(e, command, reply))
            finally:                                                                                                        # we don't stop
                if len(reply) > 0:                                                                                          # if there is a reply
                    value = reply.rstrip('#').strip()                                                                       # return the value
                    if command == 'CMS':
                        self.logger.info('Return Value Add Model Point: {0}'.format(reply))
                else:                                                                                                       #
                    if command in self.app.mount.BLIND_COMMANDS:                                                            # these are the commands, which do not expect a return value
                        value = ''                                                                                          # nothing
                    else:
                        value = '0'
        else:                                                                                                               # from here we doing the simulation for 10micron mounts commands
            if command == 'Gev':                                                                                            # which are special, but only for the most important for MW to run
                value = str(self.ascom.SiteElevation)
            elif command == 'Gmte':
                value = '0125'
            elif command == 'Gt':
                value = transform.Transform.decimalToDegree(self.ascom.SiteLatitude, True, False)
            elif command == 'Gg':
                lon = transform.Transform.decimalToDegree(self.ascom.SiteLongitude, True, False)
                if lon[0] == '-':                                                                                           # due to compatibility to LX200 protocol east is negative
                    lon1 = lon.replace('-', '+')                                                                            # change that
                else:
                    lon1 = lon.replace('+', '-')                                                                            # and vice versa
                value = lon1
            elif command.startswith('Sz'):
                self.value_azimuth = float(command[2:5]) + float(command[6:8]) / 60
            elif command.startswith('Sa'):
                self.value_altitude = float(command[2:5]) + float(command[6:8]) / 60
            elif command == 'MS':
                self.ascom.Tracking = False
                self.ascom.SlewToAltAzAsync(self.value_azimuth, self.value_altitude)
                self.ascom.Tracking = True
            elif command == 'MA':
                self.ascom.Tracking = False
                self.ascom.SlewToAltAzAsync(self.value_azimuth, self.value_altitude)
                self.ascom.Tracking = False
            elif command == 'GS':
                value = transform.Transform.decimalToDegree(self.ascom.SiderealTime, False, False)
            elif command == 'GRTMP':
                value = '10.0'
            elif command == 'Ginfo':
                raJnow = self.ascom.RightAscension
                decJnow = self.ascom.Declination
                az = self.ascom.Azimuth
                alt = self.ascom.Altitude
                if self.ascom.Slewing:
                    stat = 6
                else:
                    if self.ascom.Tracking:
                        stat = 0
                    else:
                        stat = 7
                jd = self.ascom.SiderealTime + 2440587.5
                if self.ascom.SideOfPier == 0:
                    pierside = 'E'
                else:
                    pierside = 'W'
                if self.ascom.Slewing:
                    slew = 1
                else:
                    slew = 0
                value = '{0},{1},{2},{3},{4},{5},{6},{7}#'.format(raJnow, decJnow, pierside, az, alt, jd, stat, slew)
            elif command == 'PO':
                self.ascom.Unpark()
            elif command == 'hP':
                self.ascom.Park()
            elif command == 'AP':
                self.ascom.Tracking = True
            elif command == 'RT9':
                self.ascom.Tracking = False
            elif command == 'GTMP1':
                value = '10.0'
            elif command == 'GRPRS':
                value = '990.0'
            elif command == 'Guaf':
                value = '0'
            elif command == 'GMs':
                value = '15'
            elif command == 'Gh':
                value = '90'
            elif command == 'Go':
                value = '00'
            elif command == 'Gdat':
                value = '0'
            elif command in ['GVD', 'GVN', 'GVP', 'GVT', 'GVZ']:
                value = 'Simulation'
            elif command == 'GREF':
                value = '1'
            elif command == 'CMS':
                value = 'V'
            elif command == 'getalst':
                value = '-1'
            elif command == 'GDUTV':
                value = '1,1'
            else:
                value = '0'
        self.sendCommandLock.release()
        return value

    def setupDriver(self):
        try:
            if self.ascom:
                self.ascom.connected = False
            self.chooser = Dispatch('ASCOM.Utilities.Chooser')
            self.chooser.DeviceType = 'Telescope'
            self.driverName = self.chooser.Choose(self.driverName)
            self.logger.info('driver chosen:{0}'.format(self.driverName))
            if self.driverName == 'ASCOM.FrejvallGM.Telescope' or self.driverName == 'ASCOM.tenmicron_mount.Telescope':
                self.driver_real = True
            else:
                self.driver_real = False
            self.connected = False                                                                                          # run the driver setup dialog
        except Exception as e:                                                                                              # general exception
            self.app.messageQueue.put('Driver Exception in setupMount')                                                     # write to gui
            self.logger.error('general exception:{0}'.format(e))
            self.connected = False                                                                                          # set to disconnected
        finally:                                                                                                            # won't stop the program, continue
            return



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################


# matplotlib
import matplotlib
from PyQt5.QtCore import *
# import for the PyQt5 Framework
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

from widgets import popup_dialog_ui

matplotlib.use('Qt5Agg')
from matplotlib import pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas


class xMplCanvas(FigureCanvas):

    def __init__(self, parent=None, xval=None, yval=None, titel=None):
        self.fig = plt.figure(dpi=75)
        rect = self.fig.patch
        rect.set_facecolor((25/256, 25/256, 25/256))
        self.axes = self.fig.add_subplot(111)
        self.axes.grid(True, color='white')
        # We want the axes cleared every time plot() is called
        # self.axes.hold(False)
        self.axes.set_axis_bgcolor((48/256, 48/256, 48/256))
        self.axes.tick_params(axis='x', colors='white')
        self.axes.tick_params(axis='y', colors='white')
        plt.rcParams['toolbar'] = 'None'
        plt.rcParams['axes.titlesize'] = 'large'
        plt.rcParams['axes.labelsize'] = 'medium'
        plt.tight_layout(rect=[0.1, 0.1, 0.95, 0.95])
        self.compute_initial_figure()
        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)
        FigureCanvas.updateGeometry(self)

    def compute_initial_figure(self):
        pass


class PopupData(xMplCanvas):

    def compute_initial_figure(self):
        plt.xlabel('DEC Error (arcsec)', color='white')
        plt.ylabel('Altitude (degree)', color='white')
        plt.title('Altitude over Declination Error', color='white')
        self.axes.plot()


class MyPopup(QWidget):
    def __init__(self, ):
        QWidget.__init__(self)
        self.moving = False
        self.offset = None
        self.ui = popup_dialog_ui.Ui_PopupDialog()
        self.ui.setupUi(self)
        self.initUI()
        #self.setGeometry(QRect(100, 100, 600, 600))
        l = QVBoxLayout(self.ui.widgetPlot)
        sc = PopupData(self.ui.widgetPlot)
        l.addWidget(sc)
        self.show()

    def mousePressEvent(self, mouseEvent):
        self.modifiers = mouseEvent.modifiers()
        if mouseEvent.button() == Qt.LeftButton:
            self.moving = True
            self.offset = mouseEvent.pos()

    def mouseMoveEvent(self, mouseEvent):
        if self.moving:
            cursor = QCursor()
            self.move(cursor.pos() - self.offset)

    def mouseReleaseEvent(self, mouseEvent):
        if self.moving:
            cursor = QCursor()
            self.move(cursor.pos() - self.offset)
            self.moving = False

    def initUI(self):
        self.setWindowFlags(self.windowFlags() | Qt.FramelessWindowHint)
        self.setMouseTracking(True)
        darkPalette = QPalette()
        darkPalette.setColor(QPalette.Window, QColor(32, 32, 32))
        darkPalette.setColor(QPalette.WindowText, QColor(192, 192, 192))
        darkPalette.setColor(QPalette.Base, QColor(25, 25, 25))
        darkPalette.setColor(QPalette.AlternateBase, QColor(53, 53, 53))
        darkPalette.setColor(QPalette.ToolTipBase, QColor(255, 255, 255))
        darkPalette.setColor(QPalette.ToolTipText, QColor(255, 255, 255))
        darkPalette.setColor(QPalette.Text, QColor(32, 144, 192))
        darkPalette.setColor(QPalette.Button, QColor(24, 24, 24))
        darkPalette.setColor(QPalette.ButtonText, QColor(192, 192, 192))
        darkPalette.setColor(QPalette.BrightText, QColor(255, 0, 0))
        darkPalette.setColor(QPalette.Highlight, QColor(42, 130, 218))
        darkPalette.setColor(QPalette.HighlightedText, QColor(0, 0, 0))
        self.setPalette(darkPalette)
        palette = QPalette()
        palette.setColor(QPalette.Foreground, QColor(32, 144, 192))
        palette.setColor(QPalette.Background, QColor(53, 53, 53))
        self.ui.windowTitle.setPalette(palette)




# from distutils.core import setup
from setuptools import setup
import platform

setup(
    name='mountwizzard',
    version='2.5.13.1',
    packages=[
        'mountwizzard',
        'mountwizzard.analyse',
        'mountwizzard.astrometry',
        'mountwizzard.automation',
        'mountwizzard.baseclasses',
        'mountwizzard.camera',
        'mountwizzard.dome',
        'mountwizzard.environment',
        'mountwizzard.gui',
        'mountwizzard.indi',
        'mountwizzard.modeling',
        'mountwizzard.mount',
        'mountwizzard.relays',
        'mountwizzard.remote',
        'mountwizzard.widgets'
    ],
    python_requires='~=3.5',
    install_requires=[
        # 'PyQt5>=5.6',                   # problem on ubuntu, can't be installed via pip, should be done with apt-get install
        'matplotlib>=1.5.3',            # sudo apt-get install libfreetype6-dev might be needed
        # 'pypiwin32>=219',             # not useful for linux
        'pyfits>=3.4',
        'wakeonlan>=0.2.2',
        'requests'
    ]
    + (['PyQt5>=5.6'] if "Darwin" == platform.system() else [])
    + (['PyQt5>=5.6'] if "Windows" == platform.system() else [])
    + (['pypiwin32>=219'] if "Windows" == platform.system() else [])
    ,
    url='https://pypi.python.org/pypi/mountwizzard',
    license='APL 2.0',
    author='mw',
    author_email='michael@wuertenberger.org',
    description='tooling for a 10micron mount',
)

if platform.system() == 'Linux':
    print('#############################################')
    print('### Important hint:                       ###')
    print('### you have to install PYQT5 manually    ###')
    print('### sudo apt-get install pyqt5            ###')
    print('### There might be the need to install    ###')
    print('### libfreetype6-dev manually as well     ###')
    print('### sudo apt-get install libfreetype6-dev ###')
    print('#############################################')




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'coordinate_dialog_ui.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CoordinateDialog(object):
    def setupUi(self, CoordinateDialog):
        CoordinateDialog.setObjectName("CoordinateDialog")
        CoordinateDialog.resize(791, 639)
        font = QtGui.QFont()
        font.setFamily("Arial")
        CoordinateDialog.setFont(font)
        self.modelPointsPlot = QtWidgets.QGraphicsView(CoordinateDialog)
        self.modelPointsPlot.setGeometry(QtCore.QRect(10, 40, 771, 371))
        self.modelPointsPlot.viewport().setProperty("cursor", QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.modelPointsPlot.setAcceptDrops(False)
        self.modelPointsPlot.setAutoFillBackground(True)
        self.modelPointsPlot.setFrameShadow(QtWidgets.QFrame.Plain)
        self.modelPointsPlot.setInteractive(False)
        self.modelPointsPlot.setSceneRect(QtCore.QRectF(0.0, 0.0, 769.0, 369.0))
        self.modelPointsPlot.setObjectName("modelPointsPlot")
        self.modellingLog = QtWidgets.QTextBrowser(CoordinateDialog)
        self.modellingLog.setGeometry(QtCore.QRect(10, 450, 771, 181))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(8)
        self.modellingLog.setFont(font)
        self.modellingLog.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.modellingLog.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.modellingLog.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.modellingLog.setAcceptRichText(False)
        self.modellingLog.setObjectName("modellingLog")
        self.le_telescopeAzimut = QtWidgets.QLineEdit(CoordinateDialog)
        self.le_telescopeAzimut.setGeometry(QtCore.QRect(50, 10, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.le_telescopeAzimut.setFont(font)
        self.le_telescopeAzimut.setMouseTracking(False)
        self.le_telescopeAzimut.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_telescopeAzimut.setAcceptDrops(False)
        self.le_telescopeAzimut.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopeAzimut.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopeAzimut.setReadOnly(True)
        self.le_telescopeAzimut.setObjectName("le_telescopeAzimut")
        self.le_telescopeAltitude = QtWidgets.QLineEdit(CoordinateDialog)
        self.le_telescopeAltitude.setGeometry(QtCore.QRect(210, 10, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.le_telescopeAltitude.setFont(font)
        self.le_telescopeAltitude.setMouseTracking(False)
        self.le_telescopeAltitude.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_telescopeAltitude.setAcceptDrops(False)
        self.le_telescopeAltitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_telescopeAltitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_telescopeAltitude.setReadOnly(True)
        self.le_telescopeAltitude.setObjectName("le_telescopeAltitude")
        self.label_9 = QtWidgets.QLabel(CoordinateDialog)
        self.label_9.setGeometry(QtCore.QRect(20, 10, 31, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        font.setBold(False)
        font.setWeight(50)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.label_10 = QtWidgets.QLabel(CoordinateDialog)
        self.label_10.setGeometry(QtCore.QRect(170, 10, 41, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        font.setBold(False)
        font.setWeight(50)
        self.label_10.setFont(font)
        self.label_10.setObjectName("label_10")
        self.label_109 = QtWidgets.QLabel(CoordinateDialog)
        self.label_109.setGeometry(QtCore.QRect(130, 10, 21, 20))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.label_109.setFont(font)
        self.label_109.setAlignment(QtCore.Qt.AlignCenter)
        self.label_109.setWordWrap(False)
        self.label_109.setObjectName("label_109")
        self.label_110 = QtWidgets.QLabel(CoordinateDialog)
        self.label_110.setGeometry(QtCore.QRect(290, 10, 21, 20))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.label_110.setFont(font)
        self.label_110.setAlignment(QtCore.Qt.AlignCenter)
        self.label_110.setWordWrap(False)
        self.label_110.setObjectName("label_110")
        self.label_142 = QtWidgets.QLabel(CoordinateDialog)
        self.label_142.setGeometry(QtCore.QRect(620, 10, 41, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(12)
        font.setBold(False)
        font.setWeight(50)
        self.label_142.setFont(font)
        self.label_142.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_142.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_142.setObjectName("label_142")
        self.le_SQR = QtWidgets.QLineEdit(CoordinateDialog)
        self.le_SQR.setGeometry(QtCore.QRect(660, 10, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.le_SQR.setFont(font)
        self.le_SQR.setMouseTracking(False)
        self.le_SQR.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_SQR.setAcceptDrops(False)
        self.le_SQR.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_SQR.setMaxLength(8)
        self.le_SQR.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_SQR.setReadOnly(True)
        self.le_SQR.setObjectName("le_SQR")
        self.label_87 = QtWidgets.QLabel(CoordinateDialog)
        self.label_87.setGeometry(QtCore.QRect(740, 10, 41, 20))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        self.label_87.setFont(font)
        self.label_87.setAlignment(QtCore.Qt.AlignCenter)
        self.label_87.setWordWrap(False)
        self.label_87.setObjectName("label_87")
        self.label_11 = QtWidgets.QLabel(CoordinateDialog)
        self.label_11.setGeometry(QtCore.QRect(530, 420, 41, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_11.setFont(font)
        self.label_11.setObjectName("label_11")
        self.le_modelingStatus = QtWidgets.QLineEdit(CoordinateDialog)
        self.le_modelingStatus.setGeometry(QtCore.QRect(570, 420, 61, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_modelingStatus.setFont(font)
        self.le_modelingStatus.setMouseTracking(False)
        self.le_modelingStatus.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_modelingStatus.setAcceptDrops(False)
        self.le_modelingStatus.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_modelingStatus.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_modelingStatus.setReadOnly(True)
        self.le_modelingStatus.setObjectName("le_modelingStatus")
        self.bar_modelingStatusPercent = QtWidgets.QProgressBar(CoordinateDialog)
        self.bar_modelingStatusPercent.setGeometry(QtCore.QRect(10, 420, 511, 23))
        self.bar_modelingStatusPercent.setMaximum(1000)
        self.bar_modelingStatusPercent.setProperty("value", 1)
        self.bar_modelingStatusPercent.setTextVisible(False)
        self.bar_modelingStatusPercent.setObjectName("bar_modelingStatusPercent")
        self.le_modelingStatusTime = QtWidgets.QLineEdit(CoordinateDialog)
        self.le_modelingStatusTime.setGeometry(QtCore.QRect(710, 420, 41, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_modelingStatusTime.setFont(font)
        self.le_modelingStatusTime.setMouseTracking(False)
        self.le_modelingStatusTime.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_modelingStatusTime.setAcceptDrops(False)
        self.le_modelingStatusTime.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_modelingStatusTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_modelingStatusTime.setReadOnly(True)
        self.le_modelingStatusTime.setObjectName("le_modelingStatusTime")
        self.label_12 = QtWidgets.QLabel(CoordinateDialog)
        self.label_12.setGeometry(QtCore.QRect(650, 420, 61, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_12.setFont(font)
        self.label_12.setObjectName("label_12")
        self.label_13 = QtWidgets.QLabel(CoordinateDialog)
        self.label_13.setGeometry(QtCore.QRect(760, 420, 21, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_13.setFont(font)
        self.label_13.setObjectName("label_13")
        self.checkRunTrackingWidget = QtWidgets.QCheckBox(CoordinateDialog)
        self.checkRunTrackingWidget.setGeometry(QtCore.QRect(340, 10, 101, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkRunTrackingWidget.setFont(font)
        self.checkRunTrackingWidget.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkRunTrackingWidget.setChecked(True)
        self.checkRunTrackingWidget.setObjectName("checkRunTrackingWidget")

        self.retranslateUi(CoordinateDialog)
        QtCore.QMetaObject.connectSlotsByName(CoordinateDialog)

    def retranslateUi(self, CoordinateDialog):
        _translate = QtCore.QCoreApplication.translate
        CoordinateDialog.setWindowTitle(_translate("CoordinateDialog", "Modeling Plot"))
        self.modellingLog.setHtml(_translate("CoordinateDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'MS Shell Dlg 2\'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.le_telescopeAzimut.setText(_translate("CoordinateDialog", "130,05"))
        self.le_telescopeAltitude.setText(_translate("CoordinateDialog", "80,50"))
        self.label_9.setText(_translate("CoordinateDialog", "AZ:"))
        self.label_10.setText(_translate("CoordinateDialog", "ALT:"))
        self.label_109.setText(_translate("CoordinateDialog", "°"))
        self.label_110.setText(_translate("CoordinateDialog", "°"))
        self.label_142.setText(_translate("CoordinateDialog", "SQR:"))
        self.le_SQR.setToolTip(_translate("CoordinateDialog", "shows the refraction correction status on / off"))
        self.le_SQR.setText(_translate("CoordinateDialog", "19.00"))
        self.label_87.setText(_translate("CoordinateDialog", "mpas"))
        self.label_11.setText(_translate("CoordinateDialog", "Point:"))
        self.le_modelingStatus.setToolTip(_translate("CoordinateDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_modelingStatus.setText(_translate("CoordinateDialog", "-- of --"))
        self.le_modelingStatusTime.setToolTip(_translate("CoordinateDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_modelingStatusTime.setText(_translate("CoordinateDialog", "--:--"))
        self.label_12.setText(_translate("CoordinateDialog", "est. finish"))
        self.label_13.setText(_translate("CoordinateDialog", "min"))
        self.checkRunTrackingWidget.setToolTip(_translate("CoordinateDialog", "<html><head/><body><p>Checked if you would like to see the tracking line and flip time in the window</p></body></html>"))
        self.checkRunTrackingWidget.setText(_translate("CoordinateDialog", "Show Track"))




# -*- coding: utf-8 -*-
"""

A PyQt5 (client) interface to an INDI server. This will only work
in the context of a PyQt application.

"""
import logging
from xml.etree import ElementTree
from PyQt5 import QtCore, QtNetwork
import indi.indi_xml as indiXML


class QtINDIClient(QtCore.QObject):
    logger = logging.getLogger(__name__)
    received = QtCore.pyqtSignal(object)

    def __init__(self, **kwds):
        super().__init__(**kwds)

        self.device = None
        self.message_string = ""
        self.socket = QtNetwork.QTcpSocket()
        self.socket.hostFound.connect(self.handleHostFound)
        self.socket.connected.connect(self.handleConnected)
        self.socket.readyRead.connect(self.handleReadyRead)
        self.socket.stateChanged.connect(self.handleStateChanged)
        self.socket.disconnected.connect(self.handleDisconnect)
        self.socket.error.connect(self.handleError)

    def connect(self, host, port):
        state = self.socket.state()
        print(state)
        if state == QtNetwork.QAbstractSocket.UnconnectedState:
            print('connectToHost')
            self.socket.connectToHost(host, port)
        if state == QtNetwork.QAbstractSocket.ConnectingState:
            pass
        if state == QtNetwork.QAbstractSocket.ConnectedState:
            pass

    def disconnect(self):
        self.socket.disconnectFromHost()

    def handleHostFound(self):
        pass

    def handleConnected(self):
        pass

    def handleError(self, socketError):
        print("The following error occurred: {0}".format(self.socket.errorString()))
        if socketError == QtNetwork.QAbstractSocket.RemoteHostClosedError:
            pass
        else:
            pass

    def handleStateChanged(self):
        print('State changed: {0}'.format(self.socket.state()))
        if self.socket.state() == QtNetwork.QAbstractSocket.ConnectedState:
            pass
        else:
            pass

    def handleDisconnect(self):
        print('disconnected')
        self.socket.close()

    def handleReadyRead(self):
        print('read')
        # Add starting tag if this is new message.
        if len(self.message_string) == 0:
            self.message_string = "<data>"

        # Get message from socket.
        while self.socket.bytesAvailable():

            # FIXME: This does not work with Python2.
            tmp = str(self.socket.read(1000000), "ascii")
            self.message_string += tmp

        # Add closing tag.
        self.message_string += "</data>"

        # Try and parse the message.
        try:
            messages = ElementTree.fromstring(self.message_string)
            self.message_string = ""
            for message in messages:
                xml_message = indiXML.parseETree(message)

                # Filter message is self.device is not None.
                if self.device is not None:
                    if self.device == xml_message.getAttr("device"):
                        self.received.emit(xml_message)

                # Otherwise just send them all.
                else:
                    self.received.emit(xml_message)

        # Message is incomplete, remove </data> and wait..
        except ElementTree.ParseError:
            self.message_string = self.message_string[:-7]

    def setDevice(self, device=None):
        self.device = device

    def sendMessage(self, indi_command):
        if self.socket.state() == QtNetwork.QAbstractSocket.ConnectedState:
            self.socket.write(indi_command.toXML() + b'\n')
        else:
            self.logger.warning('Socket not connected')



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################

# import basic stuff
import logging

from baseclasses.camera import MWCamera


class NoneCamera(MWCamera):
    logger = logging.getLogger(__name__)

    def __init__(self, app):
        super(NoneCamera, self).__init__(app)

    def checkAppInstall(self):
        self.appAvailable = True

    def checkAppStatus(self):
        self.appRunning = False
        self.cameraConnected = False

    def getImage(self, modelData):
        return False, 'DISCONNECTED', modelData

    def getCameraProps(self):
        suc = True
        mes = 'OK'
        canSubframe = False
        gains = ''
        sizeX = 1
        sizeY = 1
        return suc, mes, sizeX, sizeY, canSubframe, gains

    def getCameraStatus(self):
        self.cameraStatus = 'DISCONNECTED'

    def solveImage(self, modelData):
        return False, 'ERROR', modelData
